@@ -227,7 +227,7 @@ public class CallableStatement extends PreparedStatement implements
 			int localParamIndex = paramIndex - 1;
 
 			if ((paramIndex < 0) || (localParamIndex >= this.numParameters)) {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						Messages.getString("CallableStatement.11") + paramIndex //$NON-NLS-1$
 								+ Messages.getString("CallableStatement.12") + numParameters //$NON-NLS-1$
 								+ Messages.getString("CallableStatement.13"), SQLError.SQL_STATE_ILLEGAL_ARGUMENT); //$NON-NLS-1$
@@ -561,7 +561,7 @@ public class CallableStatement extends PreparedStatement implements
 				.getParameter(localParamIndex);
 
 		if (!paramDescriptor.isOut) {
-			throw new SQLException(
+			throw SQLError.createSQLException(
 					Messages.getString("CallableStatement.9") + paramIndex //$NON-NLS-1$
 							+ Messages.getString("CallableStatement.10"), //$NON-NLS-1$
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
@@ -593,7 +593,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	private void checkStreamability() throws SQLException {
 		if (this.hasOutputParams && createStreamingResultSet()) {
-			throw new SQLException(Messages.getString("CallableStatement.14"), //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("CallableStatement.14"), //$NON-NLS-1$
 					SQLError.SQL_STATE_DRIVER_NOT_CAPABLE);
 		}
 	}
@@ -609,8 +609,82 @@ public class CallableStatement extends PreparedStatement implements
 			this.outputParameterResults = null;
 		}
 	}
+
+	/**
+	 * Used to fake up some metadata when we don't have access to 
+	 * SHOW CREATE PROCEDURE or mysql.proc.
+	 * 
+	 * @throws SQLException if we can't build the metadata.
+	 */
+	private void fakeParameterTypes() throws SQLException {
+		Field[] fields = new Field[13];
+
+		fields[0] = new Field("", "PROCEDURE_CAT", Types.CHAR, 0);
+		fields[1] = new Field("", "PROCEDURE_SCHEM", Types.CHAR, 0);
+		fields[2] = new Field("", "PROCEDURE_NAME", Types.CHAR, 0);
+		fields[3] = new Field("", "COLUMN_NAME", Types.CHAR, 0);
+		fields[4] = new Field("", "COLUMN_TYPE", Types.CHAR, 0);
+		fields[5] = new Field("", "DATA_TYPE", Types.SMALLINT, 0);
+		fields[6] = new Field("", "TYPE_NAME", Types.CHAR, 0);
+		fields[7] = new Field("", "PRECISION", Types.INTEGER, 0);
+		fields[8] = new Field("", "LENGTH", Types.INTEGER, 0);
+		fields[9] = new Field("", "SCALE", Types.SMALLINT, 0);
+		fields[10] = new Field("", "RADIX", Types.SMALLINT, 0);
+		fields[11] = new Field("", "NULLABLE", Types.SMALLINT, 0);
+		fields[12] = new Field("", "REMARKS", Types.CHAR, 0);
+
+		String procName = extractProcedureName();
+
+		byte[] procNameAsBytes = null;
+
+		try {
+			procNameAsBytes = procName.getBytes("UTF-8");
+		} catch (UnsupportedEncodingException ueEx) {
+			procNameAsBytes = StringUtils.s2b(procName, this.connection);
+		}
+
+		ArrayList resultRows = new ArrayList();
+
+		for (int i = 0; i < this.parameterCount; i++) {
+			byte[][] row = new byte[13][];
+			row[0] = null; // PROCEDURE_CAT
+			row[1] = null; // PROCEDURE_SCHEM
+			row[2] = procNameAsBytes; // PROCEDURE/NAME
+			row[3] = StringUtils.s2b(String.valueOf(i), this.connection); // COLUMN_NAME
+
+			row[4] = StringUtils.s2b(String
+					.valueOf(DatabaseMetaData.procedureColumnInOut),
+					this.connection);
+
+			row[5] = StringUtils.s2b(String.valueOf(Types.VARCHAR),
+					this.connection); // DATA_TYPE
+			row[6] = StringUtils.s2b("VARCHAR", this.connection); // TYPE_NAME
+			row[7] = StringUtils.s2b(Integer.toString(65535), this.connection); // PRECISION
+			row[8] = StringUtils.s2b(Integer.toString(65535), this.connection); // LENGTH
+			row[9] = StringUtils.s2b(Integer.toString(0), this.connection); // SCALE
+			row[10] = StringUtils.s2b(Integer.toString(10), this.connection); // RADIX
+
+			row[11] = StringUtils.s2b(Integer
+					.toString(DatabaseMetaData.procedureNullableUnknown),
+					this.connection); // nullable
+
+			row[12] = null;
+
+			resultRows.add(row);
+		}
+
+		java.sql.ResultSet paramTypesRs = DatabaseMetaData.buildResultSet(
+				fields, resultRows, this.connection);
+
+		convertGetProcedureColumnsToInternalDescriptors(paramTypesRs);
+	}
 	
 	private void determineParameterTypes() throws SQLException {
+		if (this.connection.getNoAccessToProcedureBodies()) {
+			fakeParameterTypes();
+			
+			return;
+		}
 		
 		java.sql.ResultSet paramTypesRs = null;
 
@@ -780,7 +854,7 @@ public class CallableStatement extends PreparedStatement implements
 
 			return nameBuf.toString();
 		}
-		throw new SQLException(Messages.getString("CallableStatement.1"), //$NON-NLS-1$
+		throw SQLError.createSQLException(Messages.getString("CallableStatement.1"), //$NON-NLS-1$
 				SQLError.SQL_STATE_GENERAL_ERROR);
 
 	}
@@ -798,13 +872,25 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	private String fixParameterName(String paramNameIn) throws SQLException {
 		if ((paramNameIn == null) || (paramNameIn.length() == 0)) {
-			throw new SQLException(
+			throw SQLError.createSQLException(
 					((Messages.getString("CallableStatement.0") + paramNameIn) == null) //$NON-NLS-1$
 							? Messages.getString("CallableStatement.15") : Messages.getString("CallableStatement.16"), SQLError.SQL_STATE_ILLEGAL_ARGUMENT); //$NON-NLS-1$ //$NON-NLS-2$
 		}
 
+		if (this.connection.getNoAccessToProcedureBodies()) {
+			throw SQLError.createSQLException("No access to parameters by name when connection has been configured not to access procedure bodies",
+					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
+		}
 		
 		return mangleParameterName(paramNameIn);
+
+		/*
+		 * if (paramNameIn.startsWith("@")) { return paramNameIn; } else {
+		 * StringBuffer paramNameBuf = new StringBuffer("@");
+		 * paramNameBuf.append(paramNameIn);
+		 * 
+		 * return paramNameBuf.toString(); }
+		 */
 	}
 
 	/**
@@ -1212,9 +1298,13 @@ public class CallableStatement extends PreparedStatement implements
 
 	private int getNamedParamIndex(String paramName, boolean forOut)
 	throws SQLException {
+		if (this.connection.getNoAccessToProcedureBodies()) {
+			throw SQLError.createSQLException("No access to parameters by name when connection has been configured not to access procedure bodies",
+					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
+		}
 		
 		if ((paramName == null) || (paramName.length() == 0)) {
-			throw new SQLException(Messages.getString("CallableStatement.2"), //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("CallableStatement.2"), //$NON-NLS-1$
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		}
 
@@ -1222,13 +1312,13 @@ public class CallableStatement extends PreparedStatement implements
 		.getParameter(paramName);
 
 		if (this.paramInfo == null) {
-			throw new SQLException(
+			throw SQLError.createSQLException(
 					Messages.getString("CallableStatement.3") + paramName + Messages.getString("CallableStatement.4"), //$NON-NLS-1$ //$NON-NLS-2$
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		}
 
 		if (forOut && !namedParamInfo.isOut) {
-			throw new SQLException(
+			throw SQLError.createSQLException(
 					Messages.getString("CallableStatement.5") + paramName //$NON-NLS-1$
 					+ Messages.getString("CallableStatement.6"), //$NON-NLS-1$
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
@@ -1245,7 +1335,7 @@ public class CallableStatement extends PreparedStatement implements
 			}
 		}
 
-		throw new SQLException("Can't find local placeholder mapping for parameter named \"" + 
+		throw SQLError.createSQLException("Can't find local placeholder mapping for parameter named \"" + 
 				paramName + "\".", SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 	}
 
@@ -1333,11 +1423,11 @@ public class CallableStatement extends PreparedStatement implements
 
 		if (this.outputParameterResults == null) {
 			if (this.paramInfo.numberOfParameters() == 0) {
-				throw new SQLException(Messages
+				throw SQLError.createSQLException(Messages
 						.getString("CallableStatement.7"), //$NON-NLS-1$
 						SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 			}
-			throw new SQLException(Messages.getString("CallableStatement.8"), //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("CallableStatement.8"), //$NON-NLS-1$
 					SQLError.SQL_STATE_GENERAL_ERROR);
 		}
 
@@ -1608,7 +1698,7 @@ public class CallableStatement extends PreparedStatement implements
 		int rsIndex = this.parameterIndexToRsIndex[localParamIndex];
 
 		if (rsIndex == NOT_OUTPUT_PARAMETER_INDICATOR) {
-			throw new SQLException(
+			throw SQLError.createSQLException(
 					Messages.getString("CallableStatement.21") + paramIndex //$NON-NLS-1$
 							+ Messages.getString("CallableStatement.22"), //$NON-NLS-1$
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
@@ -2049,7 +2139,7 @@ public class CallableStatement extends PreparedStatement implements
 
 	public int[] executeBatch() throws SQLException {
 		if (this.hasOutputParams) {
-			throw new SQLException("Can't call executeBatch() on CallableStatement with OUTPUT parameters",
+			throw SQLError.createSQLException("Can't call executeBatch() on CallableStatement with OUTPUT parameters",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		}
 		
