@@ -56,6 +56,8 @@ public class CharsetMapping {
 	private static final Map JAVA_TO_MYSQL_CHARSET_MAP;
 
 	private static final Map JAVA_UC_TO_MYSQL_CHARSET_MAP;
+	
+	private static final Map ERROR_MESSAGE_FILE_TO_MYSQL_CHARSET_MAP;
 
 	/** Map/List of multibyte character sets (using MySQL names) */
 	private static final Map MULTIBYTE_CHARSETS;
@@ -233,7 +235,7 @@ public class CharsetMapping {
 
 		MULTIBYTE_CHARSETS = Collections.unmodifiableMap(tempMapMulti);
 
-		INDEX_TO_CHARSET = new String[99];
+		INDEX_TO_CHARSET = new String[211];
 
 		try {
 			INDEX_TO_CHARSET[1] = getJavaEncodingForMysqlEncoding("big5", null);
@@ -397,9 +399,118 @@ public class CharsetMapping {
 					null);
 			INDEX_TO_CHARSET[98] = getJavaEncodingForMysqlEncoding("eucjpms",
 					null);
+			
+			INDEX_TO_CHARSET[128] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[129] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[130] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[131] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[132] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[133] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[134] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[135] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[136] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[137] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[138] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[139] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[140] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[141] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[142] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[143] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[144] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[145] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+			INDEX_TO_CHARSET[146] = getJavaEncodingForMysqlEncoding("ucs2",
+					null);
+
+			INDEX_TO_CHARSET[192] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[193] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[194] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[195] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[196] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[197] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[198] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[199] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[200] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[201] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[202] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[203] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[204] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[205] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[206] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[207] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[208] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[209] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+			INDEX_TO_CHARSET[210] = getJavaEncodingForMysqlEncoding("utf8",
+					null);
+
 		} catch (SQLException sqlEx) {
 			// ignore, it won't happen in this case
 		}
+		
+		Map tempMap = new HashMap();
+		
+		tempMap.put("czech", "latin2");
+		tempMap.put("danish", "latin1");
+		tempMap.put("dutch", "latin1");
+		tempMap.put("english", "latin1");
+		tempMap.put("estonian", "latin7");
+		tempMap.put("french", "latin1");
+		tempMap.put("german", "latin1");
+		tempMap.put("greek", "greek");
+		tempMap.put("hungarian", "latin2");
+		tempMap.put("italian", "latin1");
+		tempMap.put("japanese", "ujis");
+		tempMap.put("japanese-sjis", "sjis");
+		tempMap.put("korean", "euckr");
+		tempMap.put("norwegian", "latin1");
+		tempMap.put("norwegian-ny", "latin1");
+		tempMap.put("polish", "latin2");
+		tempMap.put("portuguese", "latin1");
+		tempMap.put("romanian", "latin2");
+		tempMap.put("russian", "koi8r");
+		tempMap.put("serbian", "cp1250");
+		tempMap.put("slovak", "latin2");
+		tempMap.put("spanish", "latin1");
+		tempMap.put("swedish", "latin1");
+		tempMap.put("ukrainian", "koi8u");
+		
+		ERROR_MESSAGE_FILE_TO_MYSQL_CHARSET_MAP = 
+			Collections.unmodifiableMap(tempMap);
 	}
 
 	final static String getJavaEncodingForMysqlEncoding(String mysqlEncoding,
@@ -466,6 +577,65 @@ public class CharsetMapping {
 														// key
 	}
 
+	/**
+	 * Returns the character encoding for error messages returned from the
+	 * server. Doesn't return useful values other than Cp1252 until the driver
+	 * has gone through initialization phase and determined server configuration,
+	 * as not enough information is available to make an intelligent decision
+	 * until then.
+	 * 
+	 * @param conn the connection to the MySQL server
+	 * @return the Java encoding name that error messages use
+	 * @throws SQLException if determination of the character encoding fails
+	 */
+	final static String getCharacterEncodingForErrorMessages(Connection conn) throws SQLException {
+		String errorMessageFile = conn.getServerVariable("language");
+		
+		if (errorMessageFile == null || errorMessageFile.length() == 0) {
+			// punt
+			return "Cp1252";
+		}
+		
+		int endWithoutSlash = errorMessageFile.length();
+		
+		if (errorMessageFile.endsWith("/") || errorMessageFile.endsWith("\\")) {
+			endWithoutSlash--;
+		}
+			
+		int lastSlashIndex = errorMessageFile.lastIndexOf('/', endWithoutSlash - 1);
+		
+		if (lastSlashIndex == -1) {
+			lastSlashIndex = errorMessageFile.lastIndexOf('\\', endWithoutSlash - 1);
+		}
+		
+		if (lastSlashIndex == -1) {
+			lastSlashIndex = 0;
+		}
+		
+		if (lastSlashIndex == endWithoutSlash || endWithoutSlash < lastSlashIndex) {
+			// punt
+			return "Cp1252";
+		}
+		
+		errorMessageFile = errorMessageFile.substring(lastSlashIndex + 1, endWithoutSlash);
+		
+		String errorMessageEncodingMysql = (String)ERROR_MESSAGE_FILE_TO_MYSQL_CHARSET_MAP.get(errorMessageFile);
+		
+		if (errorMessageEncodingMysql == null) {
+			// punt
+			return "Cp1252";
+		}
+		
+		String javaEncoding = getJavaEncodingForMysqlEncoding(errorMessageEncodingMysql, conn);
+		
+		if (javaEncoding == null) {
+			// punt
+			return "Cp1252";
+		}
+		
+		return javaEncoding;
+	}
+	
 	final static boolean isAliasForSjis(String encoding) {
 		return ("SJIS".equalsIgnoreCase(encoding)
 				|| "WINDOWS-31J".equalsIgnoreCase(encoding)
