@@ -25,13 +25,17 @@
 package com.mysql.jdbc;
 
 import java.sql.Types;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.Map;
 
 /**
  * MysqlDefs contains many values that are needed for communication with the
  * MySQL server.
  * 
  * @author Mark Matthews
- * @version $Id: MysqlDefs.java 4723 2005-12-20 23:26:10Z mmatthews $
+ * @version $Id: MysqlDefs.java 4724 2005-12-20 23:27:01Z mmatthews $
  */
 final class MysqlDefs {
 	// ~ Static fields/initializers
@@ -45,10 +49,12 @@ final class MysqlDefs {
 
 	static final int COM_CONNECT_OUT = 20;
 
-	static final int COM_END = 28;
+	static final int COM_END = 29;
 
 	static final int COM_EXECUTE = 23;
 
+	static final int COM_FETCH = 28;
+
 	static final int COM_LONG_DATA = 24;
 
 	static final int COM_PREPARE = 22;
@@ -150,6 +156,8 @@ final class MysqlDefs {
 	 */
 	public static final int NO_CHARSET_INFO = -1;
 
+	static final byte OPEN_CURSOR_FLAG = 1;
+
 	static final int PING = 14;
 
 	static final int PROCESS_INFO = 10;
@@ -160,11 +168,11 @@ final class MysqlDefs {
 
 	static final int QUIT = 1;
 
-	static final int RELOAD = 7;
-
 	// ~ Methods
 	// ----------------------------------------------------------------
 
+	static final int RELOAD = 7;
+
 	static final int SHUTDOWN = 8;
 
 	//
@@ -319,7 +327,9 @@ final class MysqlDefs {
 	 * Maps the given MySQL type to the correct JDBC type.
 	 */
 	static int mysqlToJavaType(String mysqlType) {
-		if (mysqlType.equalsIgnoreCase("TINYINT")) { //$NON-NLS-1$
+		if (mysqlType.equalsIgnoreCase("BIT")) {
+			return mysqlToJavaType(FIELD_TYPE_BIT);
+		} else if (mysqlType.equalsIgnoreCase("TINYINT")) { //$NON-NLS-1$
 			return mysqlToJavaType(FIELD_TYPE_TINY);
 		} else if (mysqlType.equalsIgnoreCase("SMALLINT")) { //$NON-NLS-1$
 			return mysqlToJavaType(FIELD_TYPE_SHORT);
@@ -474,4 +484,109 @@ final class MysqlDefs {
 			return " Unknown MySQL Type # " + mysqlType;
 		}
 	}
+
+	private static Map mysqlToJdbcTypesMap = new HashMap();
+
+	static {
+		mysqlToJdbcTypesMap.put("BIT", new Integer(
+				mysqlToJavaType(FIELD_TYPE_BIT)));
+
+		mysqlToJdbcTypesMap.put("TINYINT", new Integer(
+				mysqlToJavaType(FIELD_TYPE_TINY)));
+		mysqlToJdbcTypesMap.put("SMALLINT", new Integer(
+				mysqlToJavaType(FIELD_TYPE_SHORT)));
+		mysqlToJdbcTypesMap.put("MEDIUMINT", new Integer(
+				mysqlToJavaType(FIELD_TYPE_INT24)));
+		mysqlToJdbcTypesMap.put("INT", new Integer(
+				mysqlToJavaType(FIELD_TYPE_LONG)));
+		mysqlToJdbcTypesMap.put("INTEGER", new Integer(
+				mysqlToJavaType(FIELD_TYPE_LONG)));
+		mysqlToJdbcTypesMap.put("BIGINT", new Integer(
+				mysqlToJavaType(FIELD_TYPE_LONGLONG)));
+		mysqlToJdbcTypesMap.put("INT24", new Integer(
+				mysqlToJavaType(FIELD_TYPE_INT24)));
+		mysqlToJdbcTypesMap.put("REAL", new Integer(
+				mysqlToJavaType(FIELD_TYPE_DOUBLE)));
+		mysqlToJdbcTypesMap.put("FLOAT", new Integer(
+				mysqlToJavaType(FIELD_TYPE_FLOAT)));
+		mysqlToJdbcTypesMap.put("DECIMAL", new Integer(
+				mysqlToJavaType(FIELD_TYPE_DECIMAL)));
+		mysqlToJdbcTypesMap.put("NUMERIC", new Integer(
+				mysqlToJavaType(FIELD_TYPE_DECIMAL)));
+		mysqlToJdbcTypesMap.put("DOUBLE", new Integer(
+				mysqlToJavaType(FIELD_TYPE_DOUBLE)));
+		mysqlToJdbcTypesMap.put("CHAR", new Integer(
+				mysqlToJavaType(FIELD_TYPE_STRING)));
+		mysqlToJdbcTypesMap.put("VARCHAR", new Integer(
+				mysqlToJavaType(FIELD_TYPE_VAR_STRING)));
+		mysqlToJdbcTypesMap.put("DATE", new Integer(
+				mysqlToJavaType(FIELD_TYPE_DATE)));
+		mysqlToJdbcTypesMap.put("TIME", new Integer(
+				mysqlToJavaType(FIELD_TYPE_TIME)));
+		mysqlToJdbcTypesMap.put("YEAR", new Integer(
+				mysqlToJavaType(FIELD_TYPE_YEAR)));
+		mysqlToJdbcTypesMap.put("TIMESTAMP", new Integer(
+				mysqlToJavaType(FIELD_TYPE_TIMESTAMP)));
+		mysqlToJdbcTypesMap.put("DATETIME", new Integer(
+				mysqlToJavaType(FIELD_TYPE_DATETIME)));
+		mysqlToJdbcTypesMap.put("TINYBLOB", new Integer(java.sql.Types.BINARY));
+		mysqlToJdbcTypesMap.put("BLOB", new Integer(
+				java.sql.Types.LONGVARBINARY));
+		mysqlToJdbcTypesMap.put("MEDIUMBLOB", new Integer(
+				java.sql.Types.LONGVARBINARY));
+		mysqlToJdbcTypesMap.put("LONGBLOB", new Integer(
+				java.sql.Types.LONGVARBINARY));
+		mysqlToJdbcTypesMap
+				.put("TINYTEXT", new Integer(java.sql.Types.VARCHAR));
+		mysqlToJdbcTypesMap
+				.put("TEXT", new Integer(java.sql.Types.LONGVARCHAR));
+		mysqlToJdbcTypesMap.put("MEDIUMTEXT", new Integer(
+				java.sql.Types.LONGVARCHAR));
+		mysqlToJdbcTypesMap.put("LONGTEXT", new Integer(
+				java.sql.Types.LONGVARCHAR));
+		mysqlToJdbcTypesMap.put("ENUM", new Integer(
+				mysqlToJavaType(FIELD_TYPE_ENUM)));
+		mysqlToJdbcTypesMap.put("SET", new Integer(
+				mysqlToJavaType(FIELD_TYPE_SET)));
+		mysqlToJdbcTypesMap.put("GEOMETRY", new Integer(
+				mysqlToJavaType(FIELD_TYPE_GEOMETRY)));
+	}
+
+	static final void appendJdbcTypeMappingQuery(StringBuffer buf, String mysqlTypeColumnName) {
+
+		buf.append("CASE ");
+		Map typesMap = new HashMap();
+		typesMap.putAll(mysqlToJdbcTypesMap);
+		typesMap.put("BINARY", new Integer(Types.BINARY));
+		typesMap.put("VARBINARY", new Integer(Types.VARBINARY));
+		
+		Iterator mysqlTypes = typesMap.keySet().iterator();
+		
+		while (mysqlTypes.hasNext()) {
+			String mysqlTypeName = (String)mysqlTypes.next();
+			buf.append(" WHEN ");
+			buf.append(mysqlTypeColumnName);
+			buf.append("='");
+			buf.append(mysqlTypeName);
+			buf.append("' THEN ");
+			buf.append(typesMap.get(mysqlTypeName));
+			
+			if (mysqlTypeName.equalsIgnoreCase("DOUBLE") ||
+					mysqlTypeName.equalsIgnoreCase("FLOAT") ||
+					mysqlTypeName.equalsIgnoreCase("DECIMAL") ||
+					mysqlTypeName.equalsIgnoreCase("NUMERIC")) {
+				buf.append(" WHEN ");
+				buf.append(mysqlTypeColumnName);
+				buf.append("='");
+				buf.append(mysqlTypeName);
+				buf.append(" unsigned' THEN ");
+				buf.append(typesMap.get(mysqlTypeName));	
+			}	
+		}
+		
+		buf.append(" ELSE ");
+		buf.append(Types.OTHER);
+		buf.append(" END ");
+		
+	}
 }
