@@ -161,7 +161,7 @@ public class NonRegisteringDriver implements java.sql.Driver {
 
 				splitValues[PORT_NUMBER_INDEX] = portAsString;
 			} else {
-				throw new SQLException(Messages
+				throw SQLError.createSQLException(Messages
 						.getString("NonRegisteringDriver.37"), //$NON-NLS-1$
 						SQLError.SQL_STATE_INVALID_CONNECTION_ATTRIBUTE);
 			}
@@ -264,7 +264,7 @@ public class NonRegisteringDriver implements java.sql.Driver {
 
 		try {
 			Connection newConn = new com.mysql.jdbc.Connection(host(props),
-					port(props), props, database(props), url, this);
+					port(props), props, database(props), url);
 
 			return newConn;
 		} catch (SQLException sqlEx) {
@@ -272,7 +272,7 @@ public class NonRegisteringDriver implements java.sql.Driver {
 			// them un-changed.
 			throw sqlEx;
 		} catch (Exception ex) {
-			throw new SQLException(Messages
+			throw SQLError.createSQLException(Messages
 					.getString("NonRegisteringDriver.17") //$NON-NLS-1$
 					+ ex.toString()
 					+ Messages.getString("NonRegisteringDriver.18"), //$NON-NLS-1$
@@ -440,11 +440,21 @@ public class NonRegisteringDriver implements java.sql.Driver {
 			return null;
 		}
 
-		if (!StringUtils.startsWithIgnoreCase(url, "jdbc:mysql://")) { //$NON-NLS-1$
+		if (!StringUtils.startsWithIgnoreCase(url, "jdbc:mysql://") && 
+				!StringUtils.startsWithIgnoreCase(url, "jdbc:mysql:mxj://")) { //$NON-NLS-1$
 
 			return null;
 		}
 
+		int beginningOfSlashes = 13;
+		
+		if (StringUtils.startsWithIgnoreCase(url, "jdbc:mysql:mxj://")) {
+			beginningOfSlashes = 17;
+			
+			urlProps.setProperty("socketFactory", 
+				"com.mysql.management.driverlaunched.ServerLauncherSocketFactory");
+		}
+		
 		/*
 		 * Parse parameters after the ? in the URL and remove them from the
 		 * original URL.
@@ -459,33 +469,38 @@ public class NonRegisteringDriver implements java.sql.Driver {
 
 			while (queryParams.hasMoreTokens()) {
 				String parameterValuePair = queryParams.nextToken();
-				
-				int indexOfEquals = StringUtils.indexOfIgnoreCase(0, parameterValuePair, "=");
+
+				int indexOfEquals = StringUtils.indexOfIgnoreCase(0,
+						parameterValuePair, "=");
 
 				String parameter = null;
 				String value = null;
-				
+
 				if (indexOfEquals != -1) {
 					parameter = parameterValuePair.substring(0, indexOfEquals);
-					
+
 					if (indexOfEquals + 1 < parameterValuePair.length()) {
 						value = parameterValuePair.substring(indexOfEquals + 1);
 					}
 				}
-				
-				if ((value != null && value.length() > 0) && 
-						(parameter != null && parameter.length() > 0)) {
+
+				if ((value != null && value.length() > 0)
+						&& (parameter != null && parameter.length() > 0)) {
 					try {
-						urlProps.put(parameter, URLDecoder.decode(value, "UTF-8"));
+						urlProps.put(parameter, URLDecoder.decode(value,
+								"UTF-8"));
 					} catch (UnsupportedEncodingException badEncoding) {
 						// punt
-						urlProps.put(parameter, value);
+						urlProps.put(parameter, URLDecoder.decode(value));
+					} catch (NoSuchMethodError nsme) {
+						// punt again
+						urlProps.put(parameter,  URLDecoder.decode(value));
 					}
 				}
 			}
 		}
 
-		url = url.substring(13);
+		url = url.substring(beginningOfSlashes);
 
 		String hostStuff = null;
 
@@ -516,21 +531,21 @@ public class NonRegisteringDriver implements java.sql.Driver {
 
 				urlProps = propTransformer.transformProperties(urlProps);
 			} catch (InstantiationException e) {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"Unable to create properties transform instance '"
 								+ propertiesTransformClassName
 								+ "' due to underlying exception: "
 								+ e.toString(),
 						SQLError.SQL_STATE_INVALID_CONNECTION_ATTRIBUTE);
 			} catch (IllegalAccessException e) {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"Unable to create properties transform instance '"
 								+ propertiesTransformClassName
 								+ "' due to underlying exception: "
 								+ e.toString(),
 						SQLError.SQL_STATE_INVALID_CONNECTION_ATTRIBUTE);
 			} catch (ClassNotFoundException e) {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"Unable to create properties transform instance '"
 								+ propertiesTransformClassName
 								+ "' due to underlying exception: "
@@ -568,14 +583,14 @@ public class NonRegisteringDriver implements java.sql.Driver {
 									"configs/" + configName + ".properties");
 
 					if (configAsStream == null) {
-						throw new SQLException(
+						throw SQLError.createSQLException(
 								"Can't find configuration template named '"
 										+ configName + "'",
 								SQLError.SQL_STATE_INVALID_CONNECTION_ATTRIBUTE);
 					}
 					configProps.load(configAsStream);
 				} catch (IOException ioEx) {
-					throw new SQLException(
+					throw SQLError.createSQLException(
 							"Unable to load configuration template '"
 									+ configName
 									+ "' due to underlying IOException: "
