@@ -33,7 +33,7 @@ import com.mysql.jdbc.profiler.ProfilerEvent;
  * Allows streaming of MySQL data.
  * 
  * @author dgan
- * @version $Id: RowDataDynamic.java 3726 2005-05-19 15:52:24Z mmatthews $
+ * @version $Id: RowDataDynamic.java 4891 2006-02-03 19:10:02Z mmatthews $
  */
 public class RowDataDynamic implements RowData {
 	// ~ Instance fields
@@ -170,7 +170,7 @@ public class RowDataDynamic implements RowData {
 									ProfilerEvent.TYPE_WARN,
 									"", //$NON-NLS-1$
 									this.owner.owningStatement == null ? "N/A" : this.owner.owningStatement.currentCatalog, //$NON-NLS-1$
-									conn.getId(),
+									this.owner.connectionId,
 									this.owner.owningStatement == null ? -1
 											: this.owner.owningStatement
 													.getId(),
