@@ -24,6 +24,8 @@
  */
 package com.mysql.jdbc;
 
+import com.mysql.jdbc.Statement.CancelTask;
+import com.mysql.jdbc.exceptions.MySQLTimeoutException;
 import com.mysql.jdbc.profiler.ProfileEventSink;
 import com.mysql.jdbc.profiler.ProfilerEvent;
 
@@ -45,6 +47,7 @@ import java.sql.ParameterMetaData;
 import java.sql.Ref;
 import java.sql.SQLException;
 import java.sql.Time;
+import java.sql.Timestamp;
 import java.sql.Types;
 
 import java.util.ArrayList;
@@ -200,24 +203,30 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 */
 	private static final byte MAX_TIME_REP_LENGTH = 13;
 
-	private static void storeTime(Buffer intoBuf, Time tm) throws SQLException {
+	private void storeTime(Buffer intoBuf, Time tm) throws SQLException {
+		
 		intoBuf.ensureCapacity(9);
 		intoBuf.writeByte((byte) 8); // length
 		intoBuf.writeByte((byte) 0); // neg flag
 		intoBuf.writeLong(0); // tm->day, not used
 
-		Calendar cal = Calendar.getInstance();
-		cal.setTime(tm);
-		intoBuf.writeByte((byte) cal.get(Calendar.HOUR_OF_DAY));
-		intoBuf.writeByte((byte) cal.get(Calendar.MINUTE));
-		intoBuf.writeByte((byte) cal.get(Calendar.SECOND));
+		Calendar sessionCalendar = getCalendarInstanceForSessionOrNew();
+		
+		synchronized (sessionCalendar) {
+			java.util.Date oldTime = sessionCalendar.getTime();
+			try {
+				sessionCalendar.setTime(tm);
+				intoBuf.writeByte((byte) sessionCalendar.get(Calendar.HOUR_OF_DAY));
+				intoBuf.writeByte((byte) sessionCalendar.get(Calendar.MINUTE));
+				intoBuf.writeByte((byte) sessionCalendar.get(Calendar.SECOND));
 
-		// intoBuf.writeLongInt(0); // tm-second_part
+				// intoBuf.writeLongInt(0); // tm-second_part
+			} finally {
+				sessionCalendar.setTime(oldTime);
+			}
+		}
 	}
 
-	/** The Calendar instance used to create date/time bindings */
-	private Calendar dateTimeBindingCal = null;
-
 	/**
 	 * Flag indicating whether or not the long parameters have been 'switched'
 	 * back to normal parameters. We can not execute() if clearParameters()
@@ -275,7 +284,8 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 * @throws SQLException
 	 *             If an error occurs
 	 */
-	public ServerPreparedStatement(Connection conn, String sql, String catalog)
+	public ServerPreparedStatement(Connection conn, String sql, String catalog,
+			int resultSetType, int resultSetConcurrency)
 			throws SQLException {
 		super(conn, catalog);
 
@@ -306,15 +316,18 @@ public class ServerPreparedStatement extends PreparedStatement {
 		try {
 			serverPrepare(sql);
 		} catch (SQLException sqlEx) {
-			realClose(false);
+			realClose(false, true);
 			// don't wrap SQLExceptions
 			throw sqlEx;
 		} catch (Exception ex) {
-			realClose(false);
+			realClose(false, true);
 
-			throw new SQLException(ex.toString(),
+			throw SQLError.createSQLException(ex.toString(),
 					SQLError.SQL_STATE_GENERAL_ERROR);
 		}
+		
+		setResultSetType(resultSetType);
+		setResultSetConcurrency(resultSetConcurrency);
 	}
 
 	/**
@@ -418,7 +431,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 	/**
 	 * @see java.sql.PreparedStatement#clearParameters()
 	 */
-	public synchronized void clearParameters() throws SQLException {
+	public void clearParameters() throws SQLException {
 		checkClosed();
 		clearParametersInternal(true);
 	}
@@ -460,7 +473,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 			return;
 		}
 		
-		realClose(true);
+		realClose(true, true);
 	}
 
 	private void dumpCloseForTestcase() {
@@ -539,7 +552,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 */
 	public synchronized int[] executeBatch() throws SQLException {
 		if (this.connection.isReadOnly()) {
-			throw new SQLException(Messages
+			throw SQLError.createSQLException(Messages
 					.getString("ServerPreparedStatement.2") //$NON-NLS-1$
 					+ Messages.getString("ServerPreparedStatement.3"), //$NON-NLS-1$
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
@@ -599,7 +612,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 								}
 
 								try {
-									updateCounts[commandIndex] = executeUpdate(false);
+									updateCounts[commandIndex] = executeUpdate(false, true);
 								} finally {
 									previousBindValuesForBatch = this.parameterBindings;
 								}
@@ -671,8 +684,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 */
 	protected com.mysql.jdbc.ResultSet executeInternal(int maxRowsToRetrieve,
 			Buffer sendPacket, boolean createStreamingResultSet,
-			boolean queryIsSelectOnly, boolean unpackFields, 
-			boolean isBatch)
+			boolean queryIsSelectOnly, boolean unpackFields, boolean isBatch)
 			throws SQLException {
 		this.numberOfExecutions++;
 
@@ -703,7 +715,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 				this.connection.getIO().dumpPacketRingBuffer();
 			}
 
-			SQLException sqlEx = new SQLException(ex.toString(),
+			SQLException sqlEx = SQLError.createSQLException(ex.toString(),
 					SQLError.SQL_STATE_GENERAL_ERROR);
 
 			if (this.connection.getDumpQueriesOnException()) {
@@ -744,7 +756,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 		checkClosed();
 		
 		if (this.parameterBindings.length == 0) {
-			throw new SQLException(Messages
+			throw SQLError.createSQLException(Messages
 					.getString("ServerPreparedStatement.8"), //$NON-NLS-1$
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		}
@@ -753,7 +765,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 		if ((parameterIndex < 0)
 				|| (parameterIndex >= this.parameterBindings.length)) {
-			throw new SQLException(Messages
+			throw SQLError.createSQLException(Messages
 					.getString("ServerPreparedStatement.9") //$NON-NLS-1$
 					+ (parameterIndex + 1)
 					+ Messages.getString("ServerPreparedStatement.10") //$NON-NLS-1$
@@ -772,14 +784,14 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 		this.parameterBindings[parameterIndex].isSet = true;
 		this.parameterBindings[parameterIndex].boundBeforeExecutionNum = this.numberOfExecutions;
-		
+
 		return this.parameterBindings[parameterIndex];
 	}
 
 	/**
 	 * @see com.mysql.jdbc.PreparedStatement#getBytes(int)
 	 */
-	synchronized byte[] getBytes(int parameterIndex) throws SQLException {
+	byte[] getBytes(int parameterIndex) throws SQLException {
 		BindValue bindValue = getBinding(parameterIndex, false);
 
 		if (bindValue.isNull) {
@@ -788,8 +800,8 @@ public class ServerPreparedStatement extends PreparedStatement {
 			throw new NotImplemented();
 		} else {
 			if (this.outByteBuffer == null) {
-				this.outByteBuffer = Buffer.allocateNew(this.connection
-						.getNetBufferLength(), false);
+				this.outByteBuffer = new Buffer(this.connection
+						.getNetBufferLength());
 			}
 
 			this.outByteBuffer.clear();
@@ -821,13 +833,14 @@ public class ServerPreparedStatement extends PreparedStatement {
 			return null;
 		}
 
-		return new ResultSetMetaData(this.resultFields);
+		return new ResultSetMetaData(this.resultFields, 
+				this.connection.getUseOldAliasMetadataBehavior());
 	}
 
 	/**
 	 * @see java.sql.PreparedStatement#getParameterMetaData()
 	 */
-	public synchronized ParameterMetaData getParameterMetaData() throws SQLException {
+	public ParameterMetaData getParameterMetaData() throws SQLException {
 		checkClosed();
 		
 		if (this.parameterMetaData == null) {
@@ -855,66 +868,61 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 * @throws SQLException
 	 *             if an error occurs
 	 */
-	protected synchronized void realClose(boolean calledExplicitly) throws SQLException {
+	protected void realClose(boolean calledExplicitly, 
+			boolean closeOpenResults) throws SQLException {
 		if (this.isClosed) {
 			return;
 		}
-		
-		
+
 		if (this.connection != null) {
-			
 			if (this.connection.getAutoGenerateTestcaseScript()) {
 				dumpCloseForTestcase();
 			}
-		
-			synchronized (this.connection) {
-				synchronized (this.connection.getMutex()) {
-									
-					//
-					// Don't communicate with the server if we're being
-					// called from the finalizer...
-					// 
-					// This will leak server resources, but if we don't do this,
-					// we'll deadlock (potentially, because there's no guarantee
-					// when, what order, and what concurrency finalizers will be
-					// called with). Well-behaved programs won't rely on finalizers
-					// to clean up their statements.
-					//
-					
-					SQLException exceptionDuringClose = null;
-					
-					
-					if (calledExplicitly) {
-						try {
-							
-							MysqlIO mysql = this.connection.getIO();
-							
-							Buffer packet = mysql.getSharedSendPacket();
-							
-							packet.writeByte((byte) MysqlDefs.COM_CLOSE_STATEMENT);
-							packet.writeLong(this.serverStatementId);
-							
-							mysql.sendCommand(MysqlDefs.COM_CLOSE_STATEMENT, null,
-									packet, true, null);
-						} catch (SQLException sqlEx) {
-							exceptionDuringClose = sqlEx;
-						}
+			
+			synchronized (this.connection.getMutex()) {
+	
+				//
+				// Don't communicate with the server if we're being
+				// called from the finalizer...
+				// 
+				// This will leak server resources, but if we don't do this,
+				// we'll deadlock (potentially, because there's no guarantee
+				// when, what order, and what concurrency finalizers will be
+				// called with). Well-behaved programs won't rely on finalizers
+				// to clean up their statements.
+				//
+				
+				SQLException exceptionDuringClose = null;
+				
+
+				if (calledExplicitly) {
+					try {
 						
+						MysqlIO mysql = this.connection.getIO();
+						
+						Buffer packet = mysql.getSharedSendPacket();
+						
+						packet.writeByte((byte) MysqlDefs.COM_CLOSE_STATEMENT);
+						packet.writeLong(this.serverStatementId);
+						
+						mysql.sendCommand(MysqlDefs.COM_CLOSE_STATEMENT, null,
+								packet, true, null);
+					} catch (SQLException sqlEx) {
+						exceptionDuringClose = sqlEx;
 					}
 					
-					
-					super.realClose(calledExplicitly);
-					
-					
-					clearParametersInternal(false);
-					this.parameterBindings = null;
-					
-					this.parameterFields = null;
-					this.resultFields = null;
-					
-					if (exceptionDuringClose != null) {
-						throw exceptionDuringClose;
-					}
+				}
+
+				super.realClose(calledExplicitly, closeOpenResults);
+
+				clearParametersInternal(false);
+				this.parameterBindings = null;
+				
+				this.parameterFields = null;
+				this.resultFields = null;
+				
+				if (exceptionDuringClose != null) {
+					throw exceptionDuringClose;
 				}
 			}
 		}
@@ -936,7 +944,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 			// don't wrap SQLExceptions
 			this.invalidationException = sqlEx;
 		} catch (Exception ex) {
-			this.invalidationException = new SQLException(ex.toString(),
+			this.invalidationException = SQLError.createSQLException(ex.toString(),
 					SQLError.SQL_STATE_GENERAL_ERROR);
 		}
 
@@ -1013,7 +1021,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 					if (this.parameterBindings[i].isLongData) {
 						if (firstFound && boundTimeToCheck != 
 							this.parameterBindings[i].boundBeforeExecutionNum) { 					
-							throw new SQLException(Messages
+							throw SQLError.createSQLException(Messages
 									.getString("ServerPreparedStatement.11") //$NON-NLS-1$
 									+ Messages.getString("ServerPreparedStatement.12"), //$NON-NLS-1$
 									SQLError.SQL_STATE_DRIVER_NOT_CAPABLE);
@@ -1030,10 +1038,11 @@ public class ServerPreparedStatement extends PreparedStatement {
 				serverResetStatement();
 			}
 
+
 			// Check bindings
 			for (int i = 0; i < this.parameterCount; i++) {
 				if (!this.parameterBindings[i].isSet) {
-					throw new SQLException(Messages
+					throw SQLError.createSQLException(Messages
 							.getString("ServerPreparedStatement.13") + (i + 1) //$NON-NLS-1$
 							+ Messages.getString("ServerPreparedStatement.14"),
 							SQLError.SQL_STATE_ILLEGAL_ARGUMENT); //$NON-NLS-1$
@@ -1064,9 +1073,28 @@ public class ServerPreparedStatement extends PreparedStatement {
 			packet.writeByte((byte) MysqlDefs.COM_EXECUTE);
 			packet.writeLong(this.serverStatementId);
 
+			boolean usingCursor = false;
+
 			if (this.connection.versionMeetsMinimum(4, 1, 2)) {
-				packet.writeByte((byte) 0); // placeholder for flags
-				packet.writeLong(1); // placeholder for parameter iterations
+				// we only create cursor-backed result sets if
+				// a) The query is a SELECT
+				// b) The server supports it
+				// c) We know it is forward-only (note this doesn't
+				// preclude updatable result sets)
+				// d) The user has set a fetch size
+				if (this.resultFields != null &&
+						this.connection.isCursorFetchEnabled()
+						&& getResultSetType() == ResultSet.TYPE_FORWARD_ONLY
+						&& getResultSetConcurrency() == ResultSet.CONCUR_READ_ONLY
+						&& getFetchSize() > 0) {
+					packet.writeByte(MysqlDefs.OPEN_CURSOR_FLAG);
+					usingCursor = true;
+				} else {
+					packet.writeByte((byte) 0); // placeholder for flags
+				}
+
+				packet.writeLong(1); // placeholder for parameter
+				                     // iterations
 			}
 
 			/* Reserve place for null-marker bytes */
@@ -1126,86 +1154,112 @@ public class ServerPreparedStatement extends PreparedStatement {
 				begin = System.currentTimeMillis();
 			}
 
-			Buffer resultPacket = mysql.sendCommand(MysqlDefs.COM_EXECUTE,
+			this.wasCancelled = false;
+			
+			CancelTask timeoutTask = null;
+
+			try {
+				if (this.timeoutInMillis != 0
+						&& this.connection.versionMeetsMinimum(5, 0, 0)) {
+					timeoutTask = new CancelTask();
+					this.connection.getCancelTimer().schedule(timeoutTask, 
+							this.timeoutInMillis);
+				}
+				
+				Buffer resultPacket = mysql.sendCommand(MysqlDefs.COM_EXECUTE,
 					null, packet, false, null);
-
+				
+				if (timeoutTask != null) {
+					timeoutTask.cancel();
+					timeoutTask = null;
+				}
+				
+				if (this.wasCancelled) {
+					this.wasCancelled = false;
+					throw new MySQLTimeoutException();
+				}
 			
-
-			this.connection.incrementNumberOfPreparedExecutes();
-
-			if (this.connection.getProfileSql()) {
-				this.eventSink = ProfileEventSink.getInstance(this.connection);
-
-				this.eventSink.consumeEvent(new ProfilerEvent(
-						ProfilerEvent.TYPE_EXECUTE, "", this.currentCatalog, //$NON-NLS-1$
-						this.connection.getId(), this.statementId, -1, System
-								.currentTimeMillis(), (int) (System
-								.currentTimeMillis() - begin), null,
-						new Throwable(), truncateQueryToLog(asSql(true))));
-			}
-
-			com.mysql.jdbc.ResultSet rs = mysql.readAllResults(this,
-					maxRowsToRetrieve, this.resultSetType,
-					this.resultSetConcurrency, createStreamingResultSet,
-					this.currentCatalog, resultPacket, true, this.fieldCount,
-					true);
-
-			
-			if (!createStreamingResultSet && 
-					this.serverNeedsResetBeforeEachExecution) {
-				serverResetStatement(); // clear any long data...
-			}
-
-			this.sendTypesToServer = false;
-			this.results = rs;
-
-			if (this.connection.getLogSlowQueries()
-					|| this.connection.getGatherPerformanceMetrics()) {
-				long elapsedTime = System.currentTimeMillis() - begin;
-
+				this.connection.incrementNumberOfPreparedExecutes();
+	
+				if (this.connection.getProfileSql()) {
+					this.eventSink = ProfileEventSink.getInstance(this.connection);
+	
+					this.eventSink.consumeEvent(new ProfilerEvent(
+							ProfilerEvent.TYPE_EXECUTE, "", this.currentCatalog, //$NON-NLS-1$
+							this.connectionId, this.statementId, -1, System
+									.currentTimeMillis(), (int) (System
+									.currentTimeMillis() - begin), null,
+							new Throwable(), truncateQueryToLog(asSql(true))));
+				}
+	
+				com.mysql.jdbc.ResultSet rs = mysql.readAllResults(this,
+						maxRowsToRetrieve, this.resultSetType,
+						this.resultSetConcurrency, createStreamingResultSet,
+						this.currentCatalog, resultPacket, true, this.fieldCount,
+						true);
+	
+				
+				if (!createStreamingResultSet && 
+						this.serverNeedsResetBeforeEachExecution) {
+					serverResetStatement(); // clear any long data...
+				}
+	
+				this.sendTypesToServer = false;
+				this.results = rs;
+	
 				if (this.connection.getLogSlowQueries()
-						&& (elapsedTime >= this.connection
-								.getSlowQueryThresholdMillis())) {
-					StringBuffer mesgBuf = new StringBuffer(
-							48 + this.originalSql.length());
-					mesgBuf.append(Messages
-							.getString("ServerPreparedStatement.15")); //$NON-NLS-1$
-					mesgBuf.append(this.connection
-							.getSlowQueryThresholdMillis());
-					mesgBuf.append(Messages
-							.getString("ServerPreparedStatement.15a")); //$NON-NLS-1$
-					mesgBuf.append(elapsedTime);
-					mesgBuf.append(Messages
-							.getString("ServerPreparedStatement.16")); //$NON-NLS-1$
-					
-					mesgBuf.append("as prepared: ");
-					mesgBuf.append(this.originalSql);
-					mesgBuf.append("\n\n with parameters bound:\n\n");
-					mesgBuf.append(asSql(true));
-
-					this.connection.getLog().logWarn(mesgBuf.toString());
-
-					if (this.connection.getExplainSlowQueries()) {
-						String queryAsString = asSql(true);
-
-						mysql.explainSlowQuery(queryAsString.getBytes(),
-								queryAsString);
+						|| this.connection.getGatherPerformanceMetrics()) {
+					long elapsedTime = System.currentTimeMillis() - begin;
+	
+					if (this.connection.getLogSlowQueries()
+							&& (elapsedTime >= this.connection
+									.getSlowQueryThresholdMillis())) {
+						StringBuffer mesgBuf = new StringBuffer(
+								48 + this.originalSql.length());
+						mesgBuf.append(Messages
+								.getString("ServerPreparedStatement.15")); //$NON-NLS-1$
+						mesgBuf.append(this.connection
+								.getSlowQueryThresholdMillis());
+						mesgBuf.append(Messages
+								.getString("ServerPreparedStatement.15a")); //$NON-NLS-1$
+						mesgBuf.append(elapsedTime);
+						mesgBuf.append(Messages
+								.getString("ServerPreparedStatement.16")); //$NON-NLS-1$
+						
+						mesgBuf.append("as prepared: ");
+						mesgBuf.append(this.originalSql);
+						mesgBuf.append("\n\n with parameters bound:\n\n");
+						mesgBuf.append(asSql(true));
+	
+						this.connection.getLog().logWarn(mesgBuf.toString());
+	
+						if (this.connection.getExplainSlowQueries()) {
+							String queryAsString = asSql(true);
+	
+							mysql.explainSlowQuery(queryAsString.getBytes(),
+									queryAsString);
+						}
+					}
+	
+					if (this.connection.getGatherPerformanceMetrics()) {
+						this.connection.registerQueryExecutionTime(elapsedTime);
 					}
 				}
-
-				if (this.connection.getGatherPerformanceMetrics()) {
-					this.connection.registerQueryExecutionTime(elapsedTime);
+				
+				if (mysql.hadWarnings()) {
+		            mysql.scanForAndThrowDataTruncation();
+		        }
+				
+				return rs;
+			} finally {
+				if (timeoutTask != null) {
+					timeoutTask.cancel();
 				}
 			}
-			
-			if (mysql.hadWarnings()) {
-	            mysql.scanForAndThrowDataTruncation();
-	        }
-			
-			return rs;
 		}
 	}
 
+
 	/**
 	 * Sends stream-type data parameters to the server.
 	 * 
@@ -1260,7 +1314,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 			} else if (value instanceof Reader) {
 				storeReader(mysql, parameterIndex, packet, (Reader) value);
 			} else {
-				throw new SQLException(Messages
+				throw SQLError.createSQLException(Messages
 						.getString("ServerPreparedStatement.18") //$NON-NLS-1$
 						+ value.getClass().getName() + "'", //$NON-NLS-1$
 						SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
@@ -1331,7 +1385,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 					this.eventSink.consumeEvent(new ProfilerEvent(
 							ProfilerEvent.TYPE_PREPARE,
 							"", this.currentCatalog, //$NON-NLS-1$
-							this.connection.getId(), this.statementId, -1,
+							this.connectionId, this.statementId, -1,
 							System.currentTimeMillis(), (int) (System
 									.currentTimeMillis() - begin), null,
 							new Throwable(), truncateQueryToLog(sql)));
@@ -1426,7 +1480,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 			} catch (SQLException sqlEx) {
 				throw sqlEx;
 			} catch (Exception ex) {
-				throw new SQLException(ex.toString(),
+				throw SQLError.createSQLException(ex.toString(),
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			} finally {
 				mysql.clearInputStream();
@@ -1485,7 +1539,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 			} else {
 				setType(binding, this.stringTypeCode);
 			}
-				
+
 			binding.value = StringUtils.fixDecimalExponent(x.toString());
 			binding.isNull = false;
 			binding.isLongData = false;
@@ -1685,7 +1739,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 		if (!this.connection.getAllowNanAndInf()
 				&& (x == Double.POSITIVE_INFINITY
 						|| x == Double.NEGATIVE_INFINITY || Double.isNaN(x))) {
-			throw new SQLException("'" + x
+			throw SQLError.createSQLException("'" + x
 					+ "' is not a valid numeric or approximate numeric value",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 
@@ -1842,7 +1896,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 */
 	public void setTime(int parameterIndex, java.sql.Time x)
 			throws SQLException {
-		setTimeInternal(parameterIndex, x, TimeZone.getDefault(), false);
+		setTimeInternal(parameterIndex, x, null, TimeZone.getDefault(), false);
 	}
 
 	/**
@@ -1862,7 +1916,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 */
 	public void setTime(int parameterIndex, java.sql.Time x, Calendar cal)
 			throws SQLException {
-		setTimeInternal(parameterIndex, x, cal.getTimeZone(), true);
+		setTimeInternal(parameterIndex, x, cal, cal.getTimeZone(), true);
 	}
 
 	/**
@@ -1881,6 +1935,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 *             if a database access error occurs
 	 */
 	public void setTimeInternal(int parameterIndex, java.sql.Time x,
+			Calendar targetCalendar,
 			TimeZone tz, boolean rollForward) throws SQLException {
 		if (x == null) {
 			setNull(parameterIndex, java.sql.Types.TIME);
@@ -1888,8 +1943,17 @@ public class ServerPreparedStatement extends PreparedStatement {
 			BindValue binding = getBinding(parameterIndex, false);
 			setType(binding, MysqlDefs.FIELD_TYPE_TIME);
 
-			binding.value = TimeUtil.changeTimezone(this.connection, x, tz,
-					this.connection.getServerTimezoneTZ(), rollForward);
+			Calendar sessionCalendar = getCalendarInstanceForSessionOrNew();
+			
+			synchronized (sessionCalendar) {
+				binding.value = TimeUtil.changeTimezone(this.connection, 
+						sessionCalendar,
+						targetCalendar,
+						x, tz,
+						this.connection.getServerTimezoneTZ(), 
+						rollForward);
+			}
+			
 			binding.isNull = false;
 			binding.isLongData = false;
 		}
@@ -1909,7 +1973,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 */
 	public void setTimestamp(int parameterIndex, java.sql.Timestamp x)
 			throws SQLException {
-		setTimestampInternal(parameterIndex, x, TimeZone.getDefault(), false);
+		setTimestampInternal(parameterIndex, x, null, TimeZone.getDefault(), false);
 	}
 
 	/**
@@ -1928,11 +1992,12 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 */
 	public void setTimestamp(int parameterIndex, java.sql.Timestamp x,
 			Calendar cal) throws SQLException {
-		setTimestampInternal(parameterIndex, x, cal.getTimeZone(), true);
+		setTimestampInternal(parameterIndex, x, cal, cal.getTimeZone(), true);
 	}
 
 	protected void setTimestampInternal(int parameterIndex,
-			java.sql.Timestamp x, TimeZone tz, boolean rollForward)
+			java.sql.Timestamp x, Calendar targetCalendar,
+			TimeZone tz, boolean rollForward)
 			throws SQLException {
 		if (x == null) {
 			setNull(parameterIndex, java.sql.Types.TIMESTAMP);
@@ -1940,8 +2005,19 @@ public class ServerPreparedStatement extends PreparedStatement {
 			BindValue binding = getBinding(parameterIndex, false);
 			setType(binding, MysqlDefs.FIELD_TYPE_DATETIME);
 
-			binding.value = TimeUtil.changeTimezone(this.connection, x, tz,
-					this.connection.getServerTimezoneTZ(), rollForward);
+			Calendar sessionCalendar = this.connection.getUseJDBCCompliantTimezoneShift() ?
+					this.connection.getUtcCalendar() : 
+						getCalendarInstanceForSessionOrNew();
+			
+			synchronized (sessionCalendar) {
+				binding.value = TimeUtil.changeTimezone(this.connection, 
+						sessionCalendar,
+						targetCalendar,
+						x, tz,
+						this.connection.getServerTimezoneTZ(), 
+						rollForward);
+			}
+			
 			binding.isNull = false;
 			binding.isLongData = false;
 		}
@@ -2053,7 +2129,8 @@ public class ServerPreparedStatement extends PreparedStatement {
 					packet.writeLenString((String) value, this.charEncoding,
 							this.connection.getServerCharacterEncoding(),
 							this.charConverter, this.connection
-									.parserKnowsUnicode());
+									.parserKnowsUnicode(),
+									this.connection);
 				} else {
 					packet.writeLenBytes(((String) value).getBytes());
 				}
@@ -2063,7 +2140,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 			
 		} catch (UnsupportedEncodingException uEE) {
-			throw new SQLException(Messages
+			throw SQLError.createSQLException(Messages
 					.getString("ServerPreparedStatement.22") //$NON-NLS-1$
 					+ this.connection.getEncoding() + "'", //$NON-NLS-1$
 					SQLError.SQL_STATE_GENERAL_ERROR);
@@ -2072,36 +2149,41 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 	private void storeDataTime412AndOlder(Buffer intoBuf, java.util.Date dt)
 			throws SQLException {
-		// This is synchronized on the connection by callers, so it is
-		// safe to lazily-instantiate this...
-		if (this.dateTimeBindingCal == null) {
-			this.dateTimeBindingCal = Calendar.getInstance();
-		}
+		
+		Calendar sessionCalendar = getCalendarInstanceForSessionOrNew();
+		
+		synchronized (sessionCalendar) {
+			java.util.Date oldTime = sessionCalendar.getTime();
+			
+			try {
+				intoBuf.ensureCapacity(8);
+				intoBuf.writeByte((byte) 7); // length
+	
+				sessionCalendar.setTime(dt);
+				
+				int year = sessionCalendar.get(Calendar.YEAR);
+				int month = sessionCalendar.get(Calendar.MONTH) + 1;
+				int date = sessionCalendar.get(Calendar.DATE);
 
-		this.dateTimeBindingCal.setTime(dt);
-
-		intoBuf.ensureCapacity(8);
-		intoBuf.writeByte((byte) 7); // length
-
-		int year = this.dateTimeBindingCal.get(Calendar.YEAR);
-		int month = this.dateTimeBindingCal.get(Calendar.MONTH) + 1;
-		int date = this.dateTimeBindingCal.get(Calendar.DATE);
-
-		intoBuf.writeInt(year);
-		intoBuf.writeByte((byte) month);
-		intoBuf.writeByte((byte) date);
-
-		if (dt instanceof java.sql.Date) {
-			intoBuf.writeByte((byte) 0);
-			intoBuf.writeByte((byte) 0);
-			intoBuf.writeByte((byte) 0);
-		} else {
-			intoBuf.writeByte((byte) this.dateTimeBindingCal
-					.get(Calendar.HOUR_OF_DAY));
-			intoBuf.writeByte((byte) this.dateTimeBindingCal
-					.get(Calendar.MINUTE));
-			intoBuf.writeByte((byte) this.dateTimeBindingCal
-					.get(Calendar.SECOND));
+				intoBuf.writeInt(year);
+				intoBuf.writeByte((byte) month);
+				intoBuf.writeByte((byte) date);
+		
+				if (dt instanceof java.sql.Date) {
+					intoBuf.writeByte((byte) 0);
+					intoBuf.writeByte((byte) 0);
+					intoBuf.writeByte((byte) 0);
+				} else {
+					intoBuf.writeByte((byte) sessionCalendar
+							.get(Calendar.HOUR_OF_DAY));
+					intoBuf.writeByte((byte) sessionCalendar
+							.get(Calendar.MINUTE));
+					intoBuf.writeByte((byte) sessionCalendar
+							.get(Calendar.SECOND));
+				}
+			} finally {
+				sessionCalendar.setTime(oldTime);
+			}
 		}
 	}
 
@@ -2116,47 +2198,61 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 	private void storeDateTime413AndNewer(Buffer intoBuf, java.util.Date dt)
 			throws SQLException {
-		// This is synchronized on the connection by callers, so it is
-		// safe to lazily-instantiate this...
-		if (this.dateTimeBindingCal == null) {
-			this.dateTimeBindingCal = Calendar.getInstance();
-		}
+		Calendar sessionCalendar = (dt instanceof Timestamp && 
+				this.connection.getUseJDBCCompliantTimezoneShift()) ? 
+				this.connection.getUtcCalendar() : getCalendarInstanceForSessionOrNew();
+		
+		synchronized (sessionCalendar) {
+			java.util.Date oldTime = sessionCalendar.getTime();
+		
+		
+			try {
+				sessionCalendar.setTime(dt);
+				
+				if (dt instanceof java.sql.Date) {
+					sessionCalendar.set(Calendar.HOUR_OF_DAY, 0);
+					sessionCalendar.set(Calendar.MINUTE, 0);
+					sessionCalendar.set(Calendar.SECOND, 0);
+				}
 
-		this.dateTimeBindingCal.setTime(dt);
+				byte length = (byte) 7;
 
-		byte length = (byte) 7;
+				intoBuf.ensureCapacity(length);
 
-		intoBuf.ensureCapacity(length);
+				if (dt instanceof java.sql.Timestamp) {
+					length = (byte) 11;
+				}
 
-		if (dt instanceof java.sql.Timestamp) {
-			length = (byte) 11;
-		}
+				intoBuf.writeByte(length); // length
 
-		intoBuf.writeByte(length); // length
+				int year = sessionCalendar.get(Calendar.YEAR);
+				int month = sessionCalendar.get(Calendar.MONTH) + 1;
+				int date = sessionCalendar.get(Calendar.DAY_OF_MONTH);
+				
+				intoBuf.writeInt(year);
+				intoBuf.writeByte((byte) month);
+				intoBuf.writeByte((byte) date);
 
-		int year = this.dateTimeBindingCal.get(Calendar.YEAR);
-		int month = this.dateTimeBindingCal.get(Calendar.MONTH) + 1;
-		int date = this.dateTimeBindingCal.get(Calendar.DATE);
+				if (dt instanceof java.sql.Date) {
+					intoBuf.writeByte((byte) 0);
+					intoBuf.writeByte((byte) 0);
+					intoBuf.writeByte((byte) 0);
+				} else {
+					intoBuf.writeByte((byte) sessionCalendar
+							.get(Calendar.HOUR_OF_DAY));
+					intoBuf.writeByte((byte) sessionCalendar
+							.get(Calendar.MINUTE));
+					intoBuf.writeByte((byte) sessionCalendar
+							.get(Calendar.SECOND));
+				}
 
-		intoBuf.writeInt(year);
-		intoBuf.writeByte((byte) month);
-		intoBuf.writeByte((byte) date);
-
-		if (dt instanceof java.sql.Date) {
-			intoBuf.writeByte((byte) 0);
-			intoBuf.writeByte((byte) 0);
-			intoBuf.writeByte((byte) 0);
-		} else {
-			intoBuf.writeByte((byte) this.dateTimeBindingCal
-					.get(Calendar.HOUR_OF_DAY));
-			intoBuf.writeByte((byte) this.dateTimeBindingCal
-					.get(Calendar.MINUTE));
-			intoBuf.writeByte((byte) this.dateTimeBindingCal
-					.get(Calendar.SECOND));
-		}
-
-		if (length == 11) {
-			intoBuf.writeLong(((java.sql.Timestamp) dt).getNanos());
+				if (length == 11) {
+					intoBuf.writeLong(((java.sql.Timestamp) dt).getNanos());
+				}
+			
+			} finally {
+				sessionCalendar.setTime(oldTime);
+			}
 		}
 	}
 
@@ -2165,26 +2261,36 @@ public class ServerPreparedStatement extends PreparedStatement {
 	//
 	private void storeReader(MysqlIO mysql, int parameterIndex, Buffer packet,
 			Reader inStream) throws SQLException {
-		int maxBytesChar = 2;
+		String forcedEncoding = this.connection.getClobCharacterEncoding();
 		
-		if (this.connection.getEncoding() != null) {
-			maxBytesChar = this.connection.getMaxBytesPerChar(
-					this.connection.getEncoding());
+		String clobEncoding = 
+			(forcedEncoding == null ? this.connection.getEncoding() : forcedEncoding);
+		
+		int maxBytesChar = 2;
 			
-			if (maxBytesChar == 1) {
-				maxBytesChar = 2; // for safety
+		if (clobEncoding != null) {
+			if (!clobEncoding.equals("UTF-16")) {
+				maxBytesChar = this.connection.getMaxBytesPerChar(clobEncoding);
+				
+				if (maxBytesChar == 1) {
+					maxBytesChar = 2; // for safety
+				}
+			} else {
+				maxBytesChar = 4;
 			}
 		}
-		
+			
 		char[] buf = new char[BLOB_STREAM_READ_BUF_SIZE / maxBytesChar];
-
+		
 		int numRead = 0;
 
 		int bytesInPacket = 0;
 		int totalBytesRead = 0;
 		int bytesReadAtLastSend = 0;
 		int packetIsFullAt = this.connection.getBlobSendChunkSize();
-
+		
+		
+		
 		try {
 			packet.clear();
 			packet.writeByte((byte) MysqlDefs.COM_LONG_DATA);
@@ -2195,9 +2301,9 @@ public class ServerPreparedStatement extends PreparedStatement {
 			
 			while ((numRead = inStream.read(buf)) != -1) {
 				readAny = true;
-				
+			
 				byte[] valueAsBytes = StringUtils.getBytes(buf, null,
-						this.connection.getEncoding(), this.connection
+						clobEncoding, this.connection
 								.getServerCharacterEncoding(), 0, numRead,
 						this.connection.parserKnowsUnicode());
 
@@ -2230,7 +2336,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 						null);
 			}
 		} catch (IOException ioEx) {
-			throw new SQLException(Messages
+			throw SQLError.createSQLException(Messages
 					.getString("ServerPreparedStatement.24") //$NON-NLS-1$
 					+ ioEx.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
 		} finally {
@@ -2297,7 +2403,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 						null);
 			}
 		} catch (IOException ioEx) {
-			throw new SQLException(Messages
+			throw SQLError.createSQLException(Messages
 					.getString("ServerPreparedStatement.25") //$NON-NLS-1$
 					+ ioEx.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
 		} finally {
@@ -2332,4 +2438,8 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 		return toStringBuf.toString();
 	}
+
+	protected long getServerStatementId() {
+		return serverStatementId;
+	}
 }
