@@ -392,7 +392,7 @@ public class StringUtils {
 
 			return b;
 		} catch (UnsupportedEncodingException uee) {
-			throw new SQLException(Messages.getString("StringUtils.5") //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("StringUtils.5") //$NON-NLS-1$
 					+ encoding + Messages.getString("StringUtils.6"),
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT); //$NON-NLS-1$
 		}
@@ -436,23 +436,29 @@ public class StringUtils {
 
 			return b;
 		} catch (UnsupportedEncodingException uee) {
-			throw new SQLException(Messages.getString("StringUtils.10") //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("StringUtils.10") //$NON-NLS-1$
 					+ encoding + Messages.getString("StringUtils.11"),
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT); //$NON-NLS-1$
 		}
 	}
 
 	public static final byte[] getBytes(char[] c, String encoding,
-			String serverEncoding, boolean parserKnowsUnicode)
+			String serverEncoding, boolean parserKnowsUnicode, Connection conn)
 			throws SQLException {
 		try {
-			SingleByteCharsetConverter converter = SingleByteCharsetConverter
-					.getInstance(encoding, null);
+			
+			SingleByteCharsetConverter converter = null;
+			
+			if (conn != null) {
+				converter = conn.getCharsetConverter(encoding);
+			} else {
+				converter = SingleByteCharsetConverter.getInstance(encoding, null);
+			}
 
 			return getBytes(c, converter, encoding, serverEncoding,
 					parserKnowsUnicode);
 		} catch (UnsupportedEncodingException uee) {
-			throw new SQLException(Messages.getString("StringUtils.0") //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("StringUtils.0") //$NON-NLS-1$
 					+ encoding + Messages.getString("StringUtils.1"),
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT); //$NON-NLS-1$
 		}
@@ -504,7 +510,7 @@ public class StringUtils {
 
 			return b;
 		} catch (UnsupportedEncodingException uee) {
-			throw new SQLException(Messages.getString("StringUtils.5") //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("StringUtils.5") //$NON-NLS-1$
 					+ encoding + Messages.getString("StringUtils.6"),
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT); //$NON-NLS-1$
 		}
@@ -558,7 +564,7 @@ public class StringUtils {
 				
 				b = new byte[length];
 				System.arraycopy(temp, 0, b, 0, length);
-				
+
 				if (!parserKnowsUnicode && (encoding.equalsIgnoreCase("SJIS") //$NON-NLS-1$
 						|| encoding.equalsIgnoreCase("BIG5") //$NON-NLS-1$
 				|| encoding.equalsIgnoreCase("GBK"))) { //$NON-NLS-1$
@@ -571,7 +577,7 @@ public class StringUtils {
 
 			return b;
 		} catch (UnsupportedEncodingException uee) {
-			throw new SQLException(Messages.getString("StringUtils.10") //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("StringUtils.10") //$NON-NLS-1$
 					+ encoding + Messages.getString("StringUtils.11"),
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT); //$NON-NLS-1$
 		}
@@ -594,16 +600,21 @@ public class StringUtils {
 	 *             if an encoding unsupported by the JVM is supplied.
 	 */
 	public static final byte[] getBytes(String s, String encoding,
-			String serverEncoding, boolean parserKnowsUnicode)
+			String serverEncoding, boolean parserKnowsUnicode, Connection conn)
 			throws SQLException {
 		try {
-			SingleByteCharsetConverter converter = SingleByteCharsetConverter
-					.getInstance(encoding, null);
+			SingleByteCharsetConverter converter = null;
+			
+			if (conn != null) {
+				converter = conn.getCharsetConverter(encoding);
+			} else {
+				converter = SingleByteCharsetConverter.getInstance(encoding, null);
+			}
 
 			return getBytes(s, converter, encoding, serverEncoding,
 					parserKnowsUnicode);
 		} catch (UnsupportedEncodingException uee) {
-			throw new SQLException(Messages.getString("StringUtils.0") //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("StringUtils.0") //$NON-NLS-1$
 					+ encoding + Messages.getString("StringUtils.1"),
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT); //$NON-NLS-1$
 		}
@@ -1156,15 +1167,32 @@ public class StringUtils {
 	 */
 	public static boolean startsWithIgnoreCaseAndWs(String searchIn,
 			String searchFor) {
+		return startsWithIgnoreCaseAndWs(searchIn, searchFor, 0);
+	}
+	
+	/**
+	 * Determines whether or not the sting 'searchIn' contains the string
+	 * 'searchFor', disregarding case and leading whitespace
+	 * 
+	 * @param searchIn
+	 *            the string to search in
+	 * @param searchFor
+	 *            the string to search for
+	 * @param beginPos
+	 *            where to start searching
+	 * 
+	 * @return true if the string starts with 'searchFor' ignoring whitespace
+	 */
+	
+	public static boolean startsWithIgnoreCaseAndWs(String searchIn,
+			String searchFor, int beginPos) {
 		if (searchIn == null) {
 			return searchFor == null;
 		}
 
-		int beginPos = 0;
-
 		int inLength = searchIn.length();
 
-		for (beginPos = 0; beginPos < inLength; beginPos++) {
+		for (; beginPos < inLength; beginPos++) {
 			if (!Character.isWhitespace(searchIn.charAt(beginPos))) {
 				break;
 			}
