@@ -27,9 +27,13 @@ package testsuite.regression;
 import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.FileInputStream;
+import java.io.BufferedOutputStream;
+import java.io.FileOutputStream;
+import java.io.IOException;
 import java.io.StringReader;
 import java.sql.Blob;
 import java.sql.Connection;
+import java.sql.DriverManager;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.Statement;
@@ -71,55 +75,55 @@ public class BlobRegressionTest extends BaseTestCase {
 	 *             ...
 	 */
 	public void testBug2670() throws Exception {
-		
-		if (isRunningOnJdk131()) {
-			return; // test not valid on JDK-1.3.1
-		}
-		
-		try {
-			byte[] blobData = new byte[32];
+		if (!isRunningOnJdk131()) {
+			try {
+				byte[] blobData = new byte[32];
 
-			for (int i = 0; i < blobData.length; i++) {
-				blobData[i] = 1;
+				for (int i = 0; i < blobData.length; i++) {
+					blobData[i] = 1;
+				}
+
+				this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug2670");
+				this.stmt
+						.executeUpdate("CREATE TABLE testBug2670(blobField LONGBLOB)");
+
+				PreparedStatement pStmt = this.conn
+						.prepareStatement("INSERT INTO testBug2670 (blobField) VALUES (?)");
+				pStmt.setBytes(1, blobData);
+				pStmt.executeUpdate();
+
+				this.rs = this.stmt
+						.executeQuery("SELECT blobField FROM testBug2670");
+				this.rs.next();
+
+				Blob blob = this.rs.getBlob(1);
+
+				//
+				// Test mid-point insertion
+				//
+				blob.setBytes(4, new byte[] { 2, 2, 2, 2 });
+
+				byte[] newBlobData = blob.getBytes(1L, (int) blob.length());
+
+				assertTrue("Blob changed length",
+						blob.length() == blobData.length);
+
+				assertTrue(
+						"New data inserted wrongly",
+						((newBlobData[3] == 2) && (newBlobData[4] == 2)
+								&& (newBlobData[5] == 2) && (newBlobData[6] == 2)));
+
+				//
+				// Test end-point insertion
+				//
+				blob.setBytes(32, new byte[] { 2, 2, 2, 2 });
+
+				assertTrue("Blob length should be 3 larger",
+						blob.length() == (blobData.length + 3));
+			} finally {
+				this.stmt
+						.executeUpdate("DROP TABLE IF EXISTS testUpdateLongBlob");
 			}
-
-			this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug2670");
-			this.stmt
-					.executeUpdate("CREATE TABLE testBug2670(blobField LONGBLOB)");
-
-			PreparedStatement pStmt = this.conn
-					.prepareStatement("INSERT INTO testBug2670 (blobField) VALUES (?)");
-			pStmt.setBytes(1, blobData);
-			pStmt.executeUpdate();
-
-			this.rs = this.stmt
-					.executeQuery("SELECT blobField FROM testBug2670");
-			this.rs.next();
-
-			Blob blob = this.rs.getBlob(1);
-
-			//
-			// Test mid-point insertion
-			//
-			blob.setBytes(4, new byte[] { 2, 2, 2, 2 });
-
-			byte[] newBlobData = blob.getBytes(1L, (int) blob.length());
-
-			assertTrue("Blob changed length", blob.length() == blobData.length);
-
-			assertTrue("New data inserted wrongly",
-					((newBlobData[3] == 2) && (newBlobData[4] == 2)
-							&& (newBlobData[5] == 2) && (newBlobData[6] == 2)));
-
-			//
-			// Test end-point insertion
-			//
-			blob.setBytes(32, new byte[] { 2, 2, 2, 2 });
-
-			assertTrue("Blob length should be 3 larger",
-					blob.length() == (blobData.length + 3));
-		} finally {
-			this.stmt.executeUpdate("DROP TABLE IF EXISTS testUpdateLongBlob");
 		}
 	}
 
@@ -254,79 +258,76 @@ public class BlobRegressionTest extends BaseTestCase {
 	 *             if the test fails.
 	 */
 	public void testBug8096() throws Exception {
-		
-		if (isRunningOnJdk131()) {
-			return; // test not valid on JDK-1.3.1
-		}
-		
-		int dataSize = 256;
+		if (!isRunningOnJdk131()) {
+			int dataSize = 256;
 
-		Properties props = new Properties();
-		props.setProperty("emulateLocators", "true");
-		Connection locatorConn = getConnectionWithProps(props);
+			Properties props = new Properties();
+			props.setProperty("emulateLocators", "true");
+			Connection locatorConn = getConnectionWithProps(props);
 
-		String createTable = "CREATE TABLE testBug8096 (ID VARCHAR(10) "
-				+ "PRIMARY KEY, DATA LONGBLOB)";
-		String select = "SELECT ID, 'DATA' AS BLOB_DATA FROM testBug8096 "
-				+ "WHERE ID = ?";
-		String insert = "INSERT INTO testBug8096 (ID, DATA) VALUES (?, '')";
+			String createTable = "CREATE TABLE testBug8096 (ID VARCHAR(10) "
+					+ "PRIMARY KEY, DATA LONGBLOB)";
+			String select = "SELECT ID, 'DATA' AS BLOB_DATA FROM testBug8096 "
+					+ "WHERE ID = ?";
+			String insert = "INSERT INTO testBug8096 (ID, DATA) VALUES (?, '')";
 
-		String id = "1";
-		byte[] testData = new byte[dataSize];
+			String id = "1";
+			byte[] testData = new byte[dataSize];
 
-		for (int i = 0; i < testData.length; i++) {
-			testData[i] = (byte) i;
-		}
-
-		try {
-			this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug8096");
-
-			this.stmt.executeUpdate(createTable);
-
-			PreparedStatement ps = locatorConn.prepareStatement(insert);
-			ps.setString(1, id);
-			ps.execute();
-
-			ps = locatorConn.prepareStatement(select);
-			ps.setString(1, id);
-
-			this.rs = ps.executeQuery();
-
-			if (this.rs.next()) {
-				Blob b = this.rs.getBlob("BLOB_DATA");
-				b.setBytes(1, testData);
+			for (int i = 0; i < testData.length; i++) {
+				testData[i] = (byte) i;
 			}
 
-			this.rs.close();
-			ps.close();
+			try {
+				this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug8096");
 
-			ps = locatorConn.prepareStatement(select);
-			ps.setString(1, id);
+				this.stmt.executeUpdate(createTable);
 
-			this.rs = ps.executeQuery();
+				PreparedStatement ps = locatorConn.prepareStatement(insert);
+				ps.setString(1, id);
+				ps.execute();
 
-			byte[] result = null;
-			if (this.rs.next()) {
-				Blob b = this.rs.getBlob("BLOB_DATA");
+				ps = locatorConn.prepareStatement(select);
+				ps.setString(1, id);
 
-				result = b.getBytes(1, dataSize - 1);
-			}
+				this.rs = ps.executeQuery();
 
-			this.rs.close();
-			ps.close();
-
-			assertNotNull(result);
-
-			for (int i = 0; i < result.length && i < testData.length; i++) {
-				// Will print out all of the values that don't match.
-				// All negative values will instead be replaced with 63.
-				if (result[i] != testData[i]) {
-					assertEquals("At position " + i, testData[i], result[i]);
+				if (this.rs.next()) {
+					Blob b = rs.getBlob("BLOB_DATA");
+					b.setBytes(1, testData);
 				}
-			}
 
-		} finally {
-			this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug8096");
+				this.rs.close();
+				ps.close();
+
+				ps = locatorConn.prepareStatement(select);
+				ps.setString(1, id);
+
+				this.rs = ps.executeQuery();
+
+				byte[] result = null;
+				if (this.rs.next()) {
+					Blob b = this.rs.getBlob("BLOB_DATA");
+
+					result = b.getBytes(1, dataSize - 1);
+				}
+
+				this.rs.close();
+				ps.close();
+
+				assertNotNull(result);
+
+				for (int i = 0; i < result.length && i < testData.length; i++) {
+					// Will print out all of the values that don't match.
+					// All negative values will instead be replaced with 63.
+					if (result[i] != testData[i]) {
+						assertEquals("At position " + i, testData[i], result[i]);
+					}
+				}
+
+			} finally {
+				this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug8096");
+			}
 		}
 	}
 
@@ -374,26 +375,28 @@ public class BlobRegressionTest extends BaseTestCase {
 
 	public void testBug10850() throws Exception {
 		String tableName = "testBug10850";
-		
+
 		createTable(tableName, "(field1 TEXT)");
-		
+
 		PreparedStatement pStmt = null;
-		
+
 		try {
-			pStmt = this.conn.prepareStatement("INSERT INTO " + 
-		
-				tableName + " VALUES (?)");
+			pStmt = this.conn.prepareStatement("INSERT INTO " +
+
+			tableName + " VALUES (?)");
 			pStmt.setCharacterStream(1, new StringReader(""), 0);
-			pStmt.executeUpdate();	
-		
-			assertEquals("0", getSingleIndexedValueWithQuery(1, "SELECT LENGTH(field1) FROM " + tableName).toString());
+			pStmt.executeUpdate();
+
+			assertEquals("0", getSingleIndexedValueWithQuery(1,
+					"SELECT LENGTH(field1) FROM " + tableName).toString());
 			this.stmt.executeUpdate("TRUNCATE TABLE " + tableName);
-		
+
 			pStmt.clearParameters();
 			pStmt.setBinaryStream(1, new ByteArrayInputStream(new byte[0]), 0);
 			pStmt.executeUpdate();
-			
-			assertEquals("0", getSingleIndexedValueWithQuery(1, "SELECT LENGTH(field1) FROM " + tableName).toString());
+
+			assertEquals("0", getSingleIndexedValueWithQuery(1,
+					"SELECT LENGTH(field1) FROM " + tableName).toString());
 			this.stmt.executeUpdate("TRUNCATE TABLE " + tableName);
 		} finally {
 			if (pStmt != null) {
@@ -401,5 +404,4 @@ public class BlobRegressionTest extends BaseTestCase {
 			}
 		}
 	}
-
 }
\ No newline at end of file
