@@ -24,6 +24,10 @@
  */
 package testsuite.regression;
 
+import java.io.ByteArrayOutputStream;
+import java.io.PrintStream;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
 import java.sql.Connection;
 import java.sql.DriverManager;
 import java.sql.ResultSet;
@@ -42,7 +46,6 @@ import com.mysql.jdbc.Driver;
 import com.mysql.jdbc.NonRegisteringDriver;
 import com.mysql.jdbc.ReplicationConnection;
 import com.mysql.jdbc.ReplicationDriver;
-import com.mysql.jdbc.log.StandardLogger;
 
 /**
  * Regression tests for Connections
@@ -157,7 +160,7 @@ public class ConnectionRegressionTest extends BaseTestCase {
 
 			conn1 = getConnectionWithProps(props); // creates a new connection
 			conn2 = getConnectionWithProps(props); // creates another new
-													// connection
+			// connection
 			conn1.setAutoCommit(false);
 			conn2.setAutoCommit(false);
 
@@ -210,9 +213,9 @@ public class ConnectionRegressionTest extends BaseTestCase {
 		if (versionMeetsMinimum(4, 1) && isAdminConnectionConfigured()) {
 			Map charsetsAndCollations = getCharacterSetsAndCollations();
 			charsetsAndCollations.remove("latin7"); // Maps to multiple Java
-													// charsets
+			// charsets
 			charsetsAndCollations.remove("ucs2"); // can't be used as a
-													// connection charset
+			// connection charset
 
 			Iterator charsets = charsetsAndCollations.keySet().iterator();
 
@@ -308,12 +311,20 @@ public class ConnectionRegressionTest extends BaseTestCase {
 		Connection killConn = getConnectionWithProps(null);
 
 		killConn.createStatement().executeUpdate("KILL " + connectionId);
-		Thread.sleep(30000);
+		Thread.sleep(2000);
 
-		try {
-			reconnectableConn.createStatement().executeQuery("SELECT 1");
-		} catch (SQLException sqlEx) {
-			; // ignore
+		SQLException caughtException = null;
+
+		int numLoops = 8;
+
+		while (caughtException == null && numLoops > 0) {
+			numLoops--;
+
+			try {
+				reconnectableConn.createStatement().executeQuery("SELECT 1");
+			} catch (SQLException sqlEx) {
+				caughtException = sqlEx;
+			}
 		}
 
 		System.out
@@ -560,7 +571,9 @@ public class ConnectionRegressionTest extends BaseTestCase {
 		if (st.hasMoreTokens()) {
 			String possibleHostOrPort = st.nextToken();
 
-			if (Character.isDigit(possibleHostOrPort.charAt(0))) {
+			if (Character.isDigit(possibleHostOrPort.charAt(0)) && 
+					(possibleHostOrPort.indexOf(".") == -1 /* IPV4 */)  &&
+					(possibleHostOrPort.indexOf("::") == -1 /* IPV6 */)) {
 				port = possibleHostOrPort;
 				host = "localhost";
 			} else {
@@ -582,7 +595,7 @@ public class ConnectionRegressionTest extends BaseTestCase {
 
 		StringBuffer newHostBuf = new StringBuffer();
 		newHostBuf.append(host);
-		newHostBuf.append(":0"); // make sure the master fails
+		newHostBuf.append(":65532"); // make sure the master fails
 		newHostBuf.append(",");
 		newHostBuf.append(host);
 		if (port != null) {
@@ -603,19 +616,23 @@ public class ConnectionRegressionTest extends BaseTestCase {
 					+ newHostBuf.toString() + "/", props);
 			failoverConnection.setAutoCommit(false);
 
+			String originalConnectionId = getSingleIndexedValueWithQuery(
+					failoverConnection, 1, "SELECT CONNECTION_ID()").toString();
+			
 			for (int i = 0; i < 49; i++) {
 				failoverConnection.createStatement().executeQuery("SELECT 1");
 			}
 
-			long begin = System.currentTimeMillis();
-
+			((com.mysql.jdbc.Connection)failoverConnection).clearHasTriedMaster();
+			
 			failoverConnection.setAutoCommit(true);
 
-			long end = System.currentTimeMillis();
-
-			assertTrue(
-					"Probably didn't try failing back to the master....check test",
-					(end - begin) > 500);
+			String newConnectionId = getSingleIndexedValueWithQuery(
+					failoverConnection, 1, "SELECT CONNECTION_ID()").toString();
+			
+			assertTrue(((com.mysql.jdbc.Connection)failoverConnection).hasTriedMaster());
+			
+			assertTrue(!newConnectionId.equals(originalConnectionId));
 
 			failoverConnection.createStatement().executeQuery("SELECT 1");
 		} finally {
@@ -651,7 +668,8 @@ public class ConnectionRegressionTest extends BaseTestCase {
 		if (st.hasMoreTokens()) {
 			String possibleHostOrPort = st.nextToken();
 
-			if (Character.isDigit(possibleHostOrPort.charAt(0))) {
+			if (possibleHostOrPort.indexOf(".") == -1
+					&& Character.isDigit(possibleHostOrPort.charAt(0))) {
 				port = possibleHostOrPort;
 				host = "localhost";
 			} else {
@@ -784,8 +802,8 @@ public class ConnectionRegressionTest extends BaseTestCase {
 						"SHOW VARIABLES LIKE 'character_set_client'");
 				assertTrue(this.rs.next());
 				String encoding = this.rs.getString(2);
-				if (!versionMeetsMinimum(5, 0, 3) &&
-					!versionMeetsMinimum(4, 1, 11)) {
+				if (!versionMeetsMinimum(5, 0, 3)
+						&& !versionMeetsMinimum(4, 1, 11)) {
 					assertEquals("sjis", encoding.toLowerCase(Locale.ENGLISH));
 				} else {
 					assertEquals("cp932", encoding.toLowerCase(Locale.ENGLISH));
@@ -797,8 +815,8 @@ public class ConnectionRegressionTest extends BaseTestCase {
 
 				String charsetToCheck = "ms932";
 
-				if (versionMeetsMinimum(5, 0, 3) || 
-					versionMeetsMinimum(4, 1, 11)) {
+				if (versionMeetsMinimum(5, 0, 3)
+						|| versionMeetsMinimum(4, 1, 11)) {
 					charsetToCheck = "windows-31j";
 				}
 
@@ -816,10 +834,10 @@ public class ConnectionRegressionTest extends BaseTestCase {
 									"create table testBug7607 (sortCol int, col1 varchar(100) ) character set sjis");
 					ms932Conn.createStatement().executeUpdate(
 							"insert into testBug7607 values(1, 0x835C)"); // standard
-																			// sjis
+					// sjis
 					ms932Conn.createStatement().executeUpdate(
 							"insert into testBug7607 values(2, 0x878A)"); // NEC
-																			// kanji
+					// kanji
 
 					this.rs = ms932Conn
 							.createStatement()
@@ -854,9 +872,16 @@ public class ConnectionRegressionTest extends BaseTestCase {
 				this.rs = shiftJisConn.createStatement().executeQuery(
 						"SELECT 'abc'");
 				assertTrue(this.rs.next());
-				assertTrue("SHIFT_JIS"
-						.equalsIgnoreCase(((com.mysql.jdbc.ResultSetMetaData) this.rs
-								.getMetaData()).getColumnCharacterSet(1)));
+
+				String charSetUC = ((com.mysql.jdbc.ResultSetMetaData) this.rs
+						.getMetaData()).getColumnCharacterSet(1).toUpperCase(
+						Locale.US);
+
+				if (isRunningOnJdk131()) {
+					assertEquals("WINDOWS-31J", charSetUC);
+				} else {
+					assertEquals("SHIFT_JIS", charSetUC);
+				}
 
 				props = new Properties();
 				props.setProperty("characterEncoding", "WINDOWS-31J");
@@ -868,8 +893,8 @@ public class ConnectionRegressionTest extends BaseTestCase {
 				assertTrue(this.rs.next());
 				encoding = this.rs.getString(2);
 
-				if (!versionMeetsMinimum(5, 0, 3) &&
-					 !versionMeetsMinimum(4, 1, 11)) {
+				if (!versionMeetsMinimum(5, 0, 3)
+						&& !versionMeetsMinimum(4, 1, 11)) {
 					assertEquals("sjis", encoding.toLowerCase(Locale.ENGLISH));
 				} else {
 					assertEquals("cp932", encoding.toLowerCase(Locale.ENGLISH));
@@ -905,9 +930,16 @@ public class ConnectionRegressionTest extends BaseTestCase {
 				this.rs = cp943Conn.createStatement().executeQuery(
 						"SELECT 'abc'");
 				assertTrue(this.rs.next());
-				assertTrue("CP943"
-						.equalsIgnoreCase(((com.mysql.jdbc.ResultSetMetaData) this.rs
-								.getMetaData()).getColumnCharacterSet(1)));
+
+				charSetUC = ((com.mysql.jdbc.ResultSetMetaData) this.rs
+						.getMetaData()).getColumnCharacterSet(1).toUpperCase(
+						Locale.US);
+
+				if (isRunningOnJdk131()) {
+					assertEquals("WINDOWS-31J", charSetUC);
+				} else {
+					assertEquals("CP943", charSetUC);
+				}
 
 			} finally {
 				if (ms932Conn != null) {
@@ -958,53 +990,82 @@ public class ConnectionRegressionTest extends BaseTestCase {
 	public void testBug8643() throws Exception {
 		if (runMultiHostTests()) {
 			Properties defaultProps = getMasterSlaveProps();
-	
+
 			defaultProps.remove(NonRegisteringDriver.HOST_PROPERTY_KEY);
 			defaultProps.remove(NonRegisteringDriver.PORT_PROPERTY_KEY);
-	
+
 			defaultProps.put("autoReconnect", "true");
 			defaultProps.put("roundRobinLoadBalance", "true");
 			defaultProps.put("failOverReadOnly", "false");
-	
+
 			Connection con = null;
 			try {
-				con = DriverManager
-						.getConnection(getMasterSlaveUrl(), defaultProps);
+				con = DriverManager.getConnection(getMasterSlaveUrl(),
+						defaultProps);
 				Statement stmt1 = con.createStatement();
-	
-				ResultSet rs1 = stmt1.executeQuery("show variables like 'port'");
+
+				ResultSet rs1 = stmt1
+						.executeQuery("show variables like 'port'");
 				rs1.next();
-	
+
 				rs1 = stmt1.executeQuery("select connection_id()");
 				rs1.next();
 				String originalConnectionId = rs1.getString(1);
-				stmt1.executeUpdate("kill " + originalConnectionId);
-				try {
-					rs1 = stmt1.executeQuery("show variables like 'port'");
-				} catch (SQLException ex) {
-					// failover and retry
-					rs1 = stmt1.executeQuery("show variables like 'port'");
+				this.stmt.executeUpdate("kill " + originalConnectionId);
+
+				int numLoops = 8;
+
+				SQLException caughtException = null;
+
+				while (caughtException == null && numLoops > 0) {
+					numLoops--;
+
+					try {
+						rs1 = stmt1.executeQuery("show variables like 'port'");
+					} catch (SQLException sqlEx) {
+						caughtException = sqlEx;
+					}
 				}
+
+				assertNotNull(caughtException);
+
+				// failover and retry
+				rs1 = stmt1.executeQuery("show variables like 'port'");
+
 				rs1.next();
-				assertTrue(!((com.mysql.jdbc.Connection) con).isMasterConnection());
-	
+				assertTrue(!((com.mysql.jdbc.Connection) con)
+						.isMasterConnection());
+
 				rs1 = stmt1.executeQuery("select connection_id()");
 				rs1.next();
 				String nextConnectionId = rs1.getString(1);
 				assertTrue(!nextConnectionId.equals(originalConnectionId));
-	
-				stmt1.executeUpdate("kill " + nextConnectionId);
-	
-				try {
-					rs1 = stmt1.executeQuery("show variables like 'port'");
-				} catch (SQLException ex) {
-					// failover and retry
-					rs1 = stmt1.executeQuery("show variables like 'port'");
+
+				this.stmt.executeUpdate("kill " + nextConnectionId);
+
+				numLoops = 8;
+
+				caughtException = null;
+
+				while (caughtException == null && numLoops > 0) {
+					numLoops--;
+
+					try {
+						rs1 = stmt1.executeQuery("show variables like 'port'");
+					} catch (SQLException sqlEx) {
+						caughtException = sqlEx;
+					}
 				}
-	
+
+				assertNotNull(caughtException);
+
+				// failover and retry
+				rs1 = stmt1.executeQuery("show variables like 'port'");
+
 				rs1.next();
-				assertTrue(((com.mysql.jdbc.Connection) con).isMasterConnection());
-	
+				assertTrue(((com.mysql.jdbc.Connection) con)
+						.isMasterConnection());
+
 			} finally {
 				if (con != null) {
 					try {
@@ -1056,7 +1117,7 @@ public class ConnectionRegressionTest extends BaseTestCase {
 			Properties props = new Properties();
 			props.setProperty("emulateUnsupportedPstmts", "false");
 			props.setProperty("useServerPrepStmts", "true");
-			
+
 			Connection bareConn = getConnectionWithProps(props);
 
 			int currentOpenStatements = ((com.mysql.jdbc.Connection) bareConn)
@@ -1069,6 +1130,10 @@ public class ConnectionRegressionTest extends BaseTestCase {
 				assertEquals(currentOpenStatements,
 						((com.mysql.jdbc.Connection) bareConn)
 								.getActiveStatementCount());
+			} finally {
+				if (bareConn != null) {
+					bareConn.close();
+				}
 			}
 		}
 	}
@@ -1097,7 +1162,8 @@ public class ConnectionRegressionTest extends BaseTestCase {
 	 * Tests fix for BUG#11259, autoReconnect ping causes exception on
 	 * connection startup.
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug11259() throws Exception {
 		Connection dsConn = null;
@@ -1111,17 +1177,18 @@ public class ConnectionRegressionTest extends BaseTestCase {
 			}
 		}
 	}
-	
+
 	/**
-	 * Tests fix for BUG#11879 -- ReplicationConnection won't
-	 * switch to slave, throws "Catalog can't be null" exception.
+	 * Tests fix for BUG#11879 -- ReplicationConnection won't switch to slave,
+	 * throws "Catalog can't be null" exception.
 	 * 
-	 * @throws Exception if the test fails
+	 * @throws Exception
+	 *             if the test fails
 	 */
 	public void testBug11879() throws Exception {
 		if (runMultiHostTests()) {
 			Connection replConn = null;
-			
+
 			try {
 				replConn = getMasterSlaveReplicationConnection();
 				replConn.setReadOnly(true);
@@ -1133,35 +1200,44 @@ public class ConnectionRegressionTest extends BaseTestCase {
 			}
 		}
 	}
-	
-	/** 
-	 * Tests fix for BUG#11976 - maxPerformance.properties
-	 * mis-spells "elideSetAutoCommits".
+
+	/**
+	 * Tests fix for BUG#11976 - maxPerformance.properties mis-spells
+	 * "elideSetAutoCommits".
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug11976() throws Exception {
+		if (isRunningOnJdk131()) {
+			return; // test not valid on JDK-1.3.1
+		}
+
 		Properties props = new Properties();
 		props.setProperty("useConfigs", "maxPerformance");
-		
+
 		Connection maxPerfConn = getConnectionWithProps(props);
-		assertEquals(true, ((com.mysql.jdbc.Connection)maxPerfConn).getElideSetAutoCommits());
+		assertEquals(true, ((com.mysql.jdbc.Connection) maxPerfConn)
+				.getElideSetAutoCommits());
 	}
-	
+
 	/**
-	 * Tests fix for BUG#12218, properties shared between master
-	 * and slave with replication connection.
+	 * Tests fix for BUG#12218, properties shared between master and slave with
+	 * replication connection.
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug12218() throws Exception {
 		if (runMultiHostTests()) {
 			Connection replConn = null;
-			
+
 			try {
 				replConn = getMasterSlaveReplicationConnection();
-				assertTrue(!((ReplicationConnection)replConn).getMasterConnection().hasSameProperties(
-						((ReplicationConnection)replConn).getSlavesConnection()));
+				assertTrue(!((ReplicationConnection) replConn)
+						.getMasterConnection().hasSameProperties(
+								((ReplicationConnection) replConn)
+										.getSlavesConnection()));
 			} finally {
 				if (replConn != null) {
 					replConn.close();
@@ -1169,67 +1245,71 @@ public class ConnectionRegressionTest extends BaseTestCase {
 			}
 		}
 	}
-	
+
 	/**
 	 * Tests fix for BUG#12229 - explainSlowQueries hangs with server-side
 	 * prepared statements.
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug12229() throws Exception {
-		createTable("testBug12229","(`int_field` integer )");  
-		this.stmt.executeUpdate("insert into testBug12229 values (123456),(1)"); 
-		
+		createTable("testBug12229", "(`int_field` integer )");
+		this.stmt.executeUpdate("insert into testBug12229 values (123456),(1)");
+
 		Properties props = new Properties();
-	 	props.put("profileSQL", "true");
-	 	props.put("slowQueryThresholdMillis", "0");
-	 	props.put("logSlowQueries", "true");
+		props.put("profileSQL", "true");
+		props.put("slowQueryThresholdMillis", "0");
+		props.put("logSlowQueries", "true");
 		props.put("explainSlowQueries", "true");
 		props.put("useServerPrepStmts", "true");
-		
+
 		Connection explainConn = getConnectionWithProps(props);
 
-		pstmt = explainConn.prepareStatement("SELECT `int_field` FROM `testBug12229` WHERE `int_field` = ?");
-		pstmt.setInt(1,1);
+		this.pstmt = explainConn
+				.prepareStatement("SELECT `int_field` FROM `testBug12229` WHERE `int_field` = ?");
+		this.pstmt.setInt(1, 1);
 
-		rs = pstmt.executeQuery();
-		assertTrue(rs.next());
-		
-		rs = pstmt.executeQuery();
-		assertTrue(rs.next());
+		this.rs = this.pstmt.executeQuery();
+		assertTrue(this.rs.next());
 
-		rs = pstmt.executeQuery();
-		assertTrue(rs.next());
+		this.rs = this.pstmt.executeQuery();
+		assertTrue(this.rs.next());
+
+		this.rs = this.pstmt.executeQuery();
+		assertTrue(this.rs.next());
 	}
-	
+
 	/**
-	 * Tests fix for BUG#12752 - Cp1251 incorrectly mapped to
-	 * win1251 for servers newer than 4.0.x.
+	 * Tests fix for BUG#12752 - Cp1251 incorrectly mapped to win1251 for
+	 * servers newer than 4.0.x.
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug12752() throws Exception {
 		Properties props = new Properties();
 		props.setProperty("characterEncoding", "Cp1251");
 		getConnectionWithProps(props).close();
 	}
-	
+
 	/**
-	 * Tests fix for BUG#12753, sessionVariables=....=....,
-	 * doesn't work as it's tokenized incorrectly.
+	 * Tests fix for BUG#12753, sessionVariables=....=...., doesn't work as it's
+	 * tokenized incorrectly.
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug12753() throws Exception {
 		if (versionMeetsMinimum(4, 1)) {
 			Properties props = new Properties();
 			props.setProperty("sessionVariables", "sql_mode=ansi");
-			
+
 			Connection sessionConn = null;
-			
+
 			try {
 				sessionConn = getConnectionWithProps(props);
-				
+
 				String sqlMode = getMysqlVariable(sessionConn, "sql_mode");
 				assertTrue(sqlMode.indexOf("ANSI") != -1);
 			} finally {
@@ -1240,30 +1320,92 @@ public class ConnectionRegressionTest extends BaseTestCase {
 			}
 		}
 	}
-	
+
 	/**
-	 * Tests fix for BUG#13453 - can't use & or = in URL configuration 
-	 * values (we now allow you to use www-form-encoding).
+	 * Tests fix for BUG#13048 - maxQuerySizeToLog is not respected.
 	 * 
-	 * @throws Exception if the test fails
+	 * @throws Exception
+	 *             if the test fails
+	 */
+	public void testBug13048() throws Exception {
+
+		Connection profileConn = null;
+		PrintStream oldErr = System.err;
+
+		try {
+			ByteArrayOutputStream bOut = new ByteArrayOutputStream();
+			System.setErr(new PrintStream(bOut));
+
+			Properties props = new Properties();
+			props.setProperty("profileSQL", "true");
+			props.setProperty("maxQuerySizeToLog", "2");
+			props.setProperty("logger", "com.mysql.jdbc.log.StandardLogger");
+
+			profileConn = getConnectionWithProps(props);
+
+			StringBuffer queryBuf = new StringBuffer("SELECT '");
+
+			for (int i = 0; i < 500; i++) {
+				queryBuf.append("a");
+			}
+
+			queryBuf.append("'");
+
+			this.rs = profileConn.createStatement().executeQuery(
+					queryBuf.toString());
+			this.rs.close();
+
+			String logString = new String(bOut.toString("ISO8859-1"));
+			assertTrue(logString.indexOf("... (truncated)") != -1);
+
+			bOut = new ByteArrayOutputStream();
+			System.setErr(new PrintStream(bOut));
+
+			this.rs = profileConn.prepareStatement(queryBuf.toString())
+					.executeQuery();
+			logString = new String(bOut.toString("ISO8859-1"));
+
+			assertTrue(logString.indexOf("... (truncated)") != -1);
+		} finally {
+			System.setErr(oldErr);
+
+			if (profileConn != null) {
+				profileConn.close();
+			}
+
+			if (this.rs != null) {
+				ResultSet toClose = this.rs;
+				this.rs = null;
+				toClose.close();
+			}
+		}
+	}
+
+	/**
+	 * Tests fix for BUG#13453 - can't use & or = in URL configuration values
+	 * (we now allow you to use www-form-encoding).
+	 * 
+	 * @throws Exception
+	 *             if the test fails
 	 */
 	public void testBug13453() throws Exception {
 		StringBuffer urlBuf = new StringBuffer(dbUrl);
-		
+
 		if (dbUrl.indexOf('?') == -1) {
 			urlBuf.append('?');
-		} else if (dbUrl.indexOf('&') != -1) {
+		} else {
 			urlBuf.append('&');
 		}
-		
+
 		urlBuf.append("sessionVariables=@testBug13453='%25%26+%3D'");
-		
+
 		Connection encodedConn = null;
-		
+
 		try {
 			encodedConn = DriverManager.getConnection(urlBuf.toString(), null);
 
-			this.rs = encodedConn.createStatement().executeQuery("SELECT @testBug13453");
+			this.rs = encodedConn.createStatement().executeQuery(
+					"SELECT @testBug13453");
 			assertTrue(this.rs.next());
 			assertEquals("%& =", this.rs.getString(1));
 		} finally {
@@ -1271,32 +1413,163 @@ public class ConnectionRegressionTest extends BaseTestCase {
 				this.rs.close();
 				this.rs = null;
 			}
-			
+
 			if (encodedConn != null) {
 				encodedConn.close();
 			}
 		}
 	}
-	
-	public void testCSC5765() throws Exception {
+
+	/**
+	 * Tests fix for BUG#15065 - Usage advisor complains about unreferenced
+	 * columns, even though they've been referenced.
+	 * 
+	 * @throws Exception
+	 *             if the test fails.
+	 */
+	public void testBug15065() throws Exception {
+		if (isRunningOnJdk131()) {
+			return; // test not valid on JDK-1.3.1
+		}
+
+		createTable("testBug15065", "(field1 int)");
+
+		this.stmt.executeUpdate("INSERT INTO testBug15065 VALUES (1)");
+
+		Connection advisorConn = null;
+		Statement advisorStmt = null;
+
+		try {
+			Properties props = new Properties();
+			props.setProperty("useUsageAdvisor", "true");
+			props.setProperty("logger", "com.mysql.jdbc.log.StandardLogger");
+
+			advisorConn = getConnectionWithProps(props);
+			advisorStmt = advisorConn.createStatement();
+
+			Method[] getMethods = ResultSet.class.getMethods();
+
+			PrintStream oldErr = System.err;
+
+			try {
+				ByteArrayOutputStream bOut = new ByteArrayOutputStream();
+				System.setErr(new PrintStream(bOut));
+				for (int j = 0; j < 2; j++) {
+					for (int i = 0; i < getMethods.length; i++) {
+						if (getMethods[i].getName().startsWith("get")
+								&& !getMethods[i].getName().equals("getURL")) {
+							Class[] parameterTypes = getMethods[i]
+									.getParameterTypes();
+
+							if (parameterTypes.length == 1
+									&& parameterTypes[0] == Integer.TYPE) {
+								if (j == 0) {
+									this.rs = advisorStmt
+											.executeQuery("SELECT COUNT(*) FROM testBug15065");
+								} else {
+									this.rs = advisorConn
+											.prepareStatement(
+													"SELECT COUNT(*) FROM testBug15065")
+											.executeQuery();
+								}
+
+								this.rs.next();
+
+								try {
+
+									getMethods[i].invoke(this.rs,
+											new Object[] { new Integer(1) });
+								} catch (InvocationTargetException invokeEx) {
+									// we don't care about bad values, just that
+									// the
+									// column gets "touched"
+									if (!invokeEx
+											.getCause()
+											.getClass()
+											.isAssignableFrom(
+													java.sql.SQLException.class)
+											&& !invokeEx
+													.getCause()
+													.getClass()
+													.getName()
+													.equals(
+															"com.mysql.jdbc.NotImplemented")) {
+										throw invokeEx;
+									}
+								}
+
+								this.rs.close();
+								this.rs = null;
+							}
+						}
+					}
+				}
+
+				String logOut = bOut.toString("ISO8859-1");
+
+				if (logOut.indexOf(".Level") != -1) {
+					return; // we ignore for warnings
+				}
+
+				assertTrue("Usage advisor complained about columns:\n\n"
+						+ logOut, logOut.indexOf("columns") == -1);
+			} finally {
+				System.setErr(oldErr);
+			}
+		} finally {
+			if (advisorConn != null) {
+				advisorConn.close();
+			}
+		}
+	}
+
+	/**
+	 * Tests fix for BUG#15544, no "dos" character set in MySQL > 4.1.0
+	 * 
+	 * @throws Exception
+	 *             if the test fails
+	 */
+	public void testBug15544() throws Exception {
 		Properties props = new Properties();
-		props.setProperty("useUnicode","true");
-		props.setProperty("characterEncoding","utf8"); 	
-		props.setProperty("characterSetResults","utf8");
-		props.setProperty("connectionCollation","utf8_bin");
-		
+		props.setProperty("characterEncoding", "Cp437");
+		Connection dosConn = null;
+
+		try {
+			dosConn = getConnectionWithProps(props);
+		} finally {
+			if (dosConn != null) {
+				dosConn.close();
+			}
+		}
+	}
+
+	public void testCSC5765() throws Exception {
+		if (isRunningOnJdk131()) {
+			return; // test not valid on JDK-1.3.1
+		}
+
+		Properties props = new Properties();
+		props.setProperty("useUnicode", "true");
+		props.setProperty("characterEncoding", "utf8");
+		props.setProperty("characterSetResults", "utf8");
+		props.setProperty("connectionCollation", "utf8_bin");
+
 		Connection utf8Conn = null;
-		
+
 		try {
 			utf8Conn = getConnectionWithProps(props);
-			this.rs = utf8Conn.createStatement().executeQuery("SHOW VARIABLES LIKE 'character_%'");
+			this.rs = utf8Conn.createStatement().executeQuery(
+					"SHOW VARIABLES LIKE 'character_%'");
 			while (this.rs.next()) {
-				System.out.println(this.rs.getString(1) + " = " + this.rs.getString(2));
+				System.out.println(this.rs.getString(1) + " = "
+						+ this.rs.getString(2));
 			}
-			
-			this.rs = utf8Conn.createStatement().executeQuery("SHOW VARIABLES LIKE 'collation_%'");
+
+			this.rs = utf8Conn.createStatement().executeQuery(
+					"SHOW VARIABLES LIKE 'collation_%'");
 			while (this.rs.next()) {
-				System.out.println(this.rs.getString(1) + " = " + this.rs.getString(2));
+				System.out.println(this.rs.getString(1) + " = "
+						+ this.rs.getString(2));
 			}
 		} finally {
 			if (utf8Conn != null) {
@@ -1304,26 +1577,26 @@ public class ConnectionRegressionTest extends BaseTestCase {
 			}
 		}
 	}
-	
-		private Connection getMasterSlaveReplicationConnection()
+
+	private Connection getMasterSlaveReplicationConnection()
 			throws SQLException {
-		
+
 		Connection replConn = new ReplicationDriver().connect(
 				getMasterSlaveUrl(), getMasterSlaveProps());
 
 		return replConn;
 	}
-	
+
 	protected String getMasterSlaveUrl() throws SQLException {
 		StringBuffer urlBuf = new StringBuffer("jdbc:mysql://");
 		Properties defaultProps = getPropertiesFromTestsuiteUrl();
 		String hostname = defaultProps
 				.getProperty(NonRegisteringDriver.HOST_PROPERTY_KEY);
-		
+
 		int colonIndex = hostname.indexOf(":");
-		
+
 		String portNumber = "3306";
-		
+
 		if (colonIndex != -1 && !hostname.startsWith(":")) {
 			portNumber = hostname.substring(colonIndex + 1);
 			hostname = hostname.substring(0, colonIndex);
@@ -1332,7 +1605,7 @@ public class ConnectionRegressionTest extends BaseTestCase {
 			hostname = "localhost";
 		} else {
 			portNumber = defaultProps
-				.getProperty(NonRegisteringDriver.PORT_PROPERTY_KEY);
+					.getProperty(NonRegisteringDriver.PORT_PROPERTY_KEY);
 		}
 
 		for (int i = 0; i < 2; i++) {
@@ -1348,13 +1621,174 @@ public class ConnectionRegressionTest extends BaseTestCase {
 
 		return urlBuf.toString();
 	}
-	
+
 	protected Properties getMasterSlaveProps() throws SQLException {
 		Properties props = getPropertiesFromTestsuiteUrl();
-		
+
 		props.remove(NonRegisteringDriver.HOST_PROPERTY_KEY);
 		props.remove(NonRegisteringDriver.PORT_PROPERTY_KEY);
-		
+
 		return props;
 	}
+
+	/**
+	 * Tests fix for BUG#15570 - ReplicationConnection incorrectly copies state,
+	 * doesn't transfer connection context correctly when transitioning between
+	 * the same read-only states.
+	 * 
+	 * (note, this test will fail if the test user doesn't have permission to
+	 * "USE 'mysql'".
+	 * 
+	 * @throws Exception
+	 *             if the test fails.
+	 */
+	public void testBug15570() throws Exception {
+		Connection replConn = null;
+
+		try {
+			replConn = getMasterSlaveReplicationConnection();
+
+			int masterConnectionId = Integer
+					.parseInt(getSingleIndexedValueWithQuery(replConn, 1,
+							"SELECT CONNECTION_ID()").toString());
+
+			replConn.setReadOnly(false);
+
+			assertEquals(masterConnectionId, Integer
+					.parseInt(getSingleIndexedValueWithQuery(replConn, 1,
+							"SELECT CONNECTION_ID()").toString()));
+
+			String currentCatalog = replConn.getCatalog();
+
+			replConn.setCatalog(currentCatalog);
+			assertEquals(currentCatalog, replConn.getCatalog());
+
+			replConn.setReadOnly(true);
+
+			int slaveConnectionId = Integer
+					.parseInt(getSingleIndexedValueWithQuery(replConn, 1,
+							"SELECT CONNECTION_ID()").toString());
+
+			// The following test is okay for now, as the chance
+			// of MySQL wrapping the connection id counter during our
+			// testsuite is very small.
+
+			assertTrue("Slave id " + slaveConnectionId
+					+ " is not newer than master id " + masterConnectionId,
+					slaveConnectionId > masterConnectionId);
+
+			assertEquals(currentCatalog, replConn.getCatalog());
+
+			String newCatalog = "mysql";
+
+			replConn.setCatalog(newCatalog);
+			assertEquals(newCatalog, replConn.getCatalog());
+
+			replConn.setReadOnly(true);
+			assertEquals(newCatalog, replConn.getCatalog());
+
+			replConn.setReadOnly(false);
+			assertEquals(masterConnectionId, Integer
+					.parseInt(getSingleIndexedValueWithQuery(replConn, 1,
+							"SELECT CONNECTION_ID()").toString()));
+		} finally {
+			if (replConn != null) {
+				replConn.close();
+			}
+		}
+	}
+
+	/**
+	 * Tests bug where downed slave caused round robin load balance not to
+	 * cycle back to first host in the list.
+	 * 
+	 * @throws Exception
+	 *             if the test fails...Note, test is timing-dependent, but
+	 *             should work in most cases.
+	 */
+	public void testBug23281() throws Exception {
+		Properties props = new Driver().parseURL(BaseTestCase.dbUrl, null);
+		props.setProperty("autoReconnect", "false");
+		props.setProperty("roundRobinLoadBalance", "true");
+		props.setProperty("failoverReadOnly", "false");
+		props.setProperty("connectTimeout", "5000");
+		
+		// Re-build the connection information
+		int firstIndexOfHost = BaseTestCase.dbUrl.indexOf("//") + 2;
+		int lastIndexOfHost = BaseTestCase.dbUrl.indexOf("/", firstIndexOfHost);
+	
+		String hostPortPair = BaseTestCase.dbUrl.substring(firstIndexOfHost,
+				lastIndexOfHost);
+	
+		StringTokenizer st = new StringTokenizer(hostPortPair, ":");
+	
+		String host = null;
+		String port = null;
+	
+		if (st.hasMoreTokens()) {
+			String possibleHostOrPort = st.nextToken();
+	
+			if (Character.isDigit(possibleHostOrPort.charAt(0)) && 
+					(possibleHostOrPort.indexOf(".") == -1 /* IPV4 */)  &&
+					(possibleHostOrPort.indexOf("::") == -1 /* IPV6 */)) {
+				port = possibleHostOrPort;
+				host = "localhost";
+			} else {
+				host = possibleHostOrPort;
+			}
+		}
+	
+		if (st.hasMoreTokens()) {
+			port = st.nextToken();
+		}
+	
+		if (host == null) {
+			host = "";
+		}
+	
+		if (port == null) {
+			port = "3306";
+		}
+	
+		StringBuffer newHostBuf = new StringBuffer();
+		
+		newHostBuf.append(host);
+		if (port != null) {
+			newHostBuf.append(":");
+			newHostBuf.append(port);
+		}
+	
+		newHostBuf.append(",");
+		//newHostBuf.append(host);
+		newHostBuf.append("192.0.2.1"); // non-exsitent machine from RFC3330 test network
+		newHostBuf.append(":65532"); // make sure the slave fails
+		
+		props.remove("PORT");
+		props.remove("HOST");
+	
+		Connection failoverConnection = null;
+	
+		try {
+			failoverConnection = getConnectionWithProps("jdbc:mysql://"
+					+ newHostBuf.toString() + "/", props);
+		
+			String originalConnectionId = getSingleIndexedValueWithQuery(
+					failoverConnection, 1, "SELECT CONNECTION_ID()").toString();
+			
+			System.out.println(originalConnectionId);
+			
+			Connection nextConnection = getConnectionWithProps("jdbc:mysql://"
+					+ newHostBuf.toString() + "/", props);
+			
+			String nextId = getSingleIndexedValueWithQuery(
+					nextConnection, 1, "SELECT CONNECTION_ID()").toString();
+			
+			System.out.println(nextId);
+			
+		} finally {
+			if (failoverConnection != null) {
+				failoverConnection.close();
+			}
+		}
+	}
 }
