@@ -60,6 +60,7 @@ import testsuite.BaseTestCase;
 
 import com.mysql.jdbc.SQLError;
 import com.mysql.jdbc.ServerPreparedStatement;
+import com.mysql.jdbc.exceptions.MySQLTimeoutException;
 
 /**
  * Regression tests for the Statement class
@@ -344,8 +345,8 @@ public class StatementRegressionTest extends BaseTestCase {
 	 */
 	private void innerBug6823(boolean continueBatchOnError) throws SQLException {
 		Properties continueBatchOnErrorProps = new Properties();
-		continueBatchOnErrorProps.setProperty("continueBatchOnError", 
-				String.valueOf(continueBatchOnError));
+		continueBatchOnErrorProps.setProperty("continueBatchOnError", String
+				.valueOf(continueBatchOnError));
 		this.conn = getConnectionWithProps(continueBatchOnErrorProps);
 		Statement statement = this.conn.createStatement();
 
@@ -619,7 +620,7 @@ public class StatementRegressionTest extends BaseTestCase {
 		if (isRunningOnJdk131()) {
 			return; // test not valid on JDK-1.3.1
 		}
-		
+
 		try {
 			this.pstmt = this.conn.prepareStatement("SELECT ?");
 			this.pstmt.setObject(1, Boolean.TRUE, Types.BOOLEAN);
@@ -632,107 +633,115 @@ public class StatementRegressionTest extends BaseTestCase {
 			}
 		}
 	}
-	
-	/** 
-	 * Tests fix for BUG#13255 - Reconnect during middle 
-	 * of executeBatch() should not happen.
+
+	/**
+	 * Tests fix for BUG#13255 - Reconnect during middle of executeBatch()
+	 * should not happen.
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug13255() throws Exception {
-		
+
 		createTable("testBug13255", "(field_1 int)");
-		
+
 		Properties props = new Properties();
 		props.setProperty("autoReconnect", "true");
-		
+
 		Connection reconnectConn = null;
 		Statement reconnectStmt = null;
 		PreparedStatement reconnectPStmt = null;
-		
+
 		try {
 			reconnectConn = getConnectionWithProps(props);
 			reconnectStmt = reconnectConn.createStatement();
-			
-			String connectionId = getSingleIndexedValueWithQuery(reconnectConn, 1, "SELECT CONNECTION_ID()").toString();
-			
+
+			String connectionId = getSingleIndexedValueWithQuery(reconnectConn,
+					1, "SELECT CONNECTION_ID()").toString();
+
 			reconnectStmt.addBatch("INSERT INTO testBug13255 VALUES (1)");
 			reconnectStmt.addBatch("INSERT INTO testBug13255 VALUES (2)");
 			reconnectStmt.addBatch("KILL " + connectionId);
-			
+
 			for (int i = 0; i < 100; i++) {
-				reconnectStmt.addBatch("INSERT INTO testBug13255 VALUES (" + i + ")");	
+				reconnectStmt.addBatch("INSERT INTO testBug13255 VALUES (" + i
+						+ ")");
 			}
-			
+
 			try {
 				reconnectStmt.executeBatch();
 			} catch (SQLException sqlEx) {
 				// We expect this...we killed the connection
 			}
-			
+
 			assertEquals(2, getRowCount("testBug13255"));
-			
+
 			this.stmt.executeUpdate("TRUNCATE TABLE testBug13255");
-			
+
 			reconnectConn.close();
-			
+
 			reconnectConn = getConnectionWithProps(props);
 
-			connectionId = getSingleIndexedValueWithQuery(reconnectConn, 1, "SELECT CONNECTION_ID()").toString();
-			
-			reconnectPStmt = reconnectConn.prepareStatement("INSERT INTO testBug13255 VALUES (?)");
+			connectionId = getSingleIndexedValueWithQuery(reconnectConn, 1,
+					"SELECT CONNECTION_ID()").toString();
+
+			reconnectPStmt = reconnectConn
+					.prepareStatement("INSERT INTO testBug13255 VALUES (?)");
 			reconnectPStmt.setInt(1, 1);
 			reconnectPStmt.addBatch();
 			reconnectPStmt.setInt(1, 2);
 			reconnectPStmt.addBatch();
 			reconnectPStmt.addBatch("KILL " + connectionId);
-			
+
 			for (int i = 3; i < 100; i++) {
 				reconnectPStmt.setInt(1, i);
 				reconnectPStmt.addBatch();
 			}
-			
+
 			try {
 				reconnectPStmt.executeBatch();
 			} catch (SQLException sqlEx) {
 				// We expect this...we killed the connection
 			}
-			
+
 			assertEquals(2, getRowCount("testBug13255"));
-			
+
 		} finally {
 			if (reconnectStmt != null) {
 				reconnectStmt.close();
 			}
-			
+
 			if (reconnectConn != null) {
 				reconnectConn.close();
 			}
 		}
 	}
-	
+
 	/**
-	 * Tests fix for BUG#15024 - Driver incorrectly closes streams passed as arguments
-	 * to PreparedStatements.
+	 * Tests fix for BUG#15024 - Driver incorrectly closes streams passed as
+	 * arguments to PreparedStatements.
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug15024() throws Exception {
 		createTable("testBug15024", "(field1 BLOB)");
-		
+
 		try {
-			this.pstmt = this.conn.prepareStatement("INSERT INTO testBug15024 VALUES (?)");
+			this.pstmt = this.conn
+					.prepareStatement("INSERT INTO testBug15024 VALUES (?)");
 			testStreamsForBug15024(false, false);
-			
+
 			Properties props = new Properties();
 			props.setProperty("useConfigs", "3-0-Compat");
-			
+
 			Connection compatConn = null;
-			
+
 			try {
 				compatConn = getConnectionWithProps(props);
 
-				this.pstmt = compatConn.prepareStatement("INSERT INTO testBug15024 VALUES (?)");
+				this.pstmt = compatConn
+						.prepareStatement("INSERT INTO testBug15024 VALUES (?)");
 				testStreamsForBug15024(true, false);
 			} finally {
 				if (compatConn != null) {
@@ -743,7 +752,7 @@ public class StatementRegressionTest extends BaseTestCase {
 			if (this.pstmt != null) {
 				PreparedStatement toClose = this.pstmt;
 				this.pstmt = null;
-				
+
 				toClose.close();
 			}
 		}
@@ -752,14 +761,16 @@ public class StatementRegressionTest extends BaseTestCase {
 	/**
 	 * PreparedStatement should call EscapeProcessor.escapeSQL?
 	 * 
-	 * @throws Exception if the test fails
+	 * @throws Exception
+	 *             if the test fails
 	 */
 	public void testBug15141() throws Exception {
 		try {
 			createTable("testBug15141", "(field1 VARCHAR(32))");
 			this.stmt.executeUpdate("INSERT INTO testBug15141 VALUES ('abc')");
-			
-			this.pstmt = this.conn.prepareStatement("select {d '1997-05-24'} FROM testBug15141");
+
+			this.pstmt = this.conn
+					.prepareStatement("select {d '1997-05-24'} FROM testBug15141");
 			this.rs = this.pstmt.executeQuery();
 			assertTrue(this.rs.next());
 			assertEquals("1997-05-24", this.rs.getString(1));
@@ -767,8 +778,9 @@ public class StatementRegressionTest extends BaseTestCase {
 			this.rs = null;
 			this.pstmt.close();
 			this.pstmt = null;
-			
-			this.pstmt = ((com.mysql.jdbc.Connection) this.conn).clientPrepareStatement("select {d '1997-05-24'} FROM testBug15141");
+
+			this.pstmt = ((com.mysql.jdbc.Connection) this.conn)
+					.clientPrepareStatement("select {d '1997-05-24'} FROM testBug15141");
 			this.rs = this.pstmt.executeQuery();
 			assertTrue(this.rs.next());
 			assertEquals("1997-05-24", this.rs.getString(1));
@@ -782,7 +794,7 @@ public class StatementRegressionTest extends BaseTestCase {
 				this.rs = null;
 				toCloseRs.close();
 			}
-			
+
 			if (this.pstmt != null) {
 				PreparedStatement toClosePstmt = this.pstmt;
 				this.pstmt = null;
@@ -790,30 +802,31 @@ public class StatementRegressionTest extends BaseTestCase {
 			}
 		}
 	}
-	
+
 	/**
-	 * Tests fix for BUG#18041 - Server-side prepared statements
-	 * don't cause truncation exceptions to be thrown.
+	 * Tests fix for BUG#18041 - Server-side prepared statements don't cause
+	 * truncation exceptions to be thrown.
 	 * 
-	 * @throws Exception if the test fails
+	 * @throws Exception
+	 *             if the test fails
 	 */
 	public void testBug18041() throws Exception {
 		if (versionMeetsMinimum(4, 1)) {
 			createTable("testBug18041", "(`a` tinyint(4) NOT NULL,"
 					+ "`b` char(4) default NULL)");
-	
+
 			Properties props = new Properties();
 			props.setProperty("jdbcCompliantTruncation", "true");
 			props.setProperty("useServerPrepStmts", "true");
-	
+
 			Connection truncConn = null;
 			PreparedStatement stm = null;
-			
+
 			try {
 				truncConn = getConnectionWithProps(props);
 
 				stm = truncConn
-					.prepareStatement("insert into testBug18041 values (?,?)");
+						.prepareStatement("insert into testBug18041 values (?,?)");
 				stm.setInt(1, 1000);
 				stm.setString(2, "nnnnnnnnnnnnnnnnnnnnnnnnnnnnnn");
 				stm.executeUpdate();
@@ -824,56 +837,56 @@ public class StatementRegressionTest extends BaseTestCase {
 				if (this.stmt != null) {
 					this.stmt.close();
 				}
-				
+
 				if (truncConn != null) {
 					truncConn.close();
 				}
 			}
-		}	
+		}
 	}
-	
-	private void testStreamsForBug15024(boolean shouldBeClosedStream, 
+
+	private void testStreamsForBug15024(boolean shouldBeClosedStream,
 			boolean shouldBeClosedReader) throws SQLException {
 		IsClosedInputStream bIn = new IsClosedInputStream(new byte[4]);
 		IsClosedReader readerIn = new IsClosedReader("abcdef");
-		
+
 		this.pstmt.setBinaryStream(1, bIn, 4);
 		this.pstmt.execute();
 		assertEquals(shouldBeClosedStream, bIn.isClosed());
-		
+
 		this.pstmt.setCharacterStream(1, readerIn, 6);
 		this.pstmt.execute();
 		assertEquals(shouldBeClosedReader, readerIn.isClosed());
-		
+
 		this.pstmt.close();
 	}
-    
+
 	class IsClosedReader extends StringReader {
 
 		boolean isClosed = false;
-		
+
 		public IsClosedReader(String arg0) {
 			super(arg0);
 		}
 
 		public void close() {
 			super.close();
-			
+
 			this.isClosed = true;
 		}
-		
+
 		public boolean isClosed() {
 			return this.isClosed;
 		}
-		
+
 	}
-	
+
 	class IsClosedInputStream extends ByteArrayInputStream {
 
 		boolean isClosed = false;
-		
+
 		public IsClosedInputStream(byte[] arg0, int arg1, int arg2) {
-			super(arg0, arg1, arg2);	
+			super(arg0, arg1, arg2);
 		}
 
 		public IsClosedInputStream(byte[] arg0) {
@@ -885,12 +898,12 @@ public class StatementRegressionTest extends BaseTestCase {
 			super.close();
 			this.isClosed = true;
 		}
-		
+
 		public boolean isClosed() {
 			return this.isClosed;
 		}
 	}
-	
+
 	/**
 	 * Tests fix for BUG#1774 -- Truncated words after double quote
 	 * 
@@ -1019,7 +1032,7 @@ public class StatementRegressionTest extends BaseTestCase {
 		if (isRunningOnJdk131()) {
 			return; // test not valid on JDK-1.3.1
 		}
-		
+
 		try {
 			this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug1934");
 			this.stmt.executeUpdate("CREATE TABLE testBug1934 (field1 INT)");
@@ -1251,6 +1264,10 @@ public class StatementRegressionTest extends BaseTestCase {
 	 *             if the test fails.
 	 */
 	public void testBug3620() throws SQLException {
+		if (isRunningOnJRockit()) {
+			// bug with their timezones
+			return;
+		}
 		
 		long epsillon = 3000; // 3 seconds time difference
 
@@ -1466,7 +1483,7 @@ public class StatementRegressionTest extends BaseTestCase {
 		if (isRunningOnJdk131()) {
 			return; // test not valid on JDK-1.3.1
 		}
-		
+
 		PreparedStatement batchStmt = null;
 
 		try {
@@ -1590,7 +1607,7 @@ public class StatementRegressionTest extends BaseTestCase {
 		if (isRunningOnJdk131()) {
 			return; // test not valid on JDK-1.3.1
 		}
-		
+
 		try {
 			this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug4510");
 			this.stmt.executeUpdate("CREATE TABLE testBug4510 ("
@@ -2183,7 +2200,7 @@ public class StatementRegressionTest extends BaseTestCase {
 		if (isRunningOnJdk131()) {
 			return; // test not valid on JDK-1.3.1
 		}
-		
+
 		Connection sjisConn = null;
 		Connection windows31JConn = null;
 
@@ -2285,7 +2302,7 @@ public class StatementRegressionTest extends BaseTestCase {
 		if (isRunningOnJdk131()) {
 			return; // test not valid on JDK-1.3.1
 		}
-		
+
 		System.out.println("Using Statement.executeUpdate()\n");
 
 		try {
@@ -2562,10 +2579,12 @@ public class StatementRegressionTest extends BaseTestCase {
 						.prepareStatement("SELECT * FROM testQuotedId WHERE col1='ABC`DEF' or col1=?");
 				pStmt.setString(1, "foo");
 				pStmt.execute();
-				
+
 				this.stmt.executeUpdate("DROP TABLE IF EXISTS testQuotedId2");
-				this.stmt.executeUpdate("CREATE TABLE testQuotedId2 (`Works?` INT)");
-				pStmt = this.conn.prepareStatement("INSERT INTO testQuotedId2 (`Works?`) VALUES (?)");
+				this.stmt
+						.executeUpdate("CREATE TABLE testQuotedId2 (`Works?` INT)");
+				pStmt = this.conn
+						.prepareStatement("INSERT INTO testQuotedId2 (`Works?`) VALUES (?)");
 				pStmt.setInt(1, 1);
 				pStmt.executeUpdate();
 			} finally {
@@ -2990,35 +3009,36 @@ public class StatementRegressionTest extends BaseTestCase {
 	}
 
 	/**
-	 * Tests fix for BUG#15383 - PreparedStatement.setObject()
-	 * serializes BigInteger as object, rather than sending as
-	 * numeric value (and is thus not complementary to .getObject()
-	 * on an UNSIGNED LONG type).
+	 * Tests fix for BUG#15383 - PreparedStatement.setObject() serializes
+	 * BigInteger as object, rather than sending as numeric value (and is thus
+	 * not complementary to .getObject() on an UNSIGNED LONG type).
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug15383() throws Exception {
-		createTable("testBug15383", "(id INTEGER UNSIGNED NOT NULL "
-				+ "AUTO_INCREMENT,value BIGINT UNSIGNED NULL DEFAULT 0,PRIMARY "
-				+ "KEY(id))ENGINE=InnoDB;");
-						
+		createTable(
+				"testBug15383",
+				"(id INTEGER UNSIGNED NOT NULL "
+						+ "AUTO_INCREMENT,value BIGINT UNSIGNED NULL DEFAULT 0,PRIMARY "
+						+ "KEY(id))ENGINE=InnoDB;");
+
 		this.stmt.executeUpdate("INSERT INTO testBug15383(value) VALUES(1)");
-		
+
 		Statement updatableStmt = this.conn.createStatement(
-				ResultSet.TYPE_FORWARD_ONLY,
-				ResultSet.CONCUR_UPDATABLE);
-		
+				ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_UPDATABLE);
+
 		try {
 			this.rs = updatableStmt.executeQuery("SELECT * from testBug15383");
-			
+
 			assertTrue(this.rs.next());
-	
+
 			Object bigIntObj = this.rs.getObject("value");
 			assertEquals("java.math.BigInteger", bigIntObj.getClass().getName());
-			
+
 			this.rs.updateObject("value", new BigInteger("3"));
 			this.rs.updateRow();
-			
+
 			assertEquals("3", this.rs.getString("value"));
 		} finally {
 			if (this.rs != null) {
@@ -3026,130 +3046,134 @@ public class StatementRegressionTest extends BaseTestCase {
 				this.rs = null;
 				toClose.close();
 			}
-			
+
 			if (updatableStmt != null) {
 				updatableStmt.close();
 			}
-		}		
+		}
 	}
-	
+
 	/**
-	 * Tests fix for BUG#17099 - Statement.getGeneratedKeys()
-	 * throws NPE when no query has been processed.
+	 * Tests fix for BUG#17099 - Statement.getGeneratedKeys() throws NPE when no
+	 * query has been processed.
 	 * 
-	 * @throws Exception if the test fails
+	 * @throws Exception
+	 *             if the test fails
 	 */
 	public void testBug17099() throws Exception {
 		if (isRunningOnJdk131()) {
 			return; // test not valid
 		}
-		
+
 		Statement newStmt = this.conn.createStatement();
 		assertNotNull(newStmt.getGeneratedKeys());
-		
+
 		PreparedStatement pStmt = this.conn.prepareStatement("SELECT 1");
 		assertNotNull(pStmt.getGeneratedKeys());
-		
+
 		if (versionMeetsMinimum(4, 1)) {
-			pStmt = ((com.mysql.jdbc.Connection)this.conn).clientPrepareStatement("SELECT 1");
+			pStmt = ((com.mysql.jdbc.Connection) this.conn)
+					.clientPrepareStatement("SELECT 1");
 			assertNotNull(pStmt.getGeneratedKeys());
 		}
 	}
-	
+
 	/**
-	 * Tests fix for BUG#17587 - clearParameters() on a closed prepared statement
-	 * causes NPE.
+	 * Tests fix for BUG#17587 - clearParameters() on a closed prepared
+	 * statement causes NPE.
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug17587() throws Exception {
 		createTable("testBug17857", "(field1 int)");
 		PreparedStatement pStmt = null;
-		
+
 		try {
-			pStmt = this.conn.prepareStatement("INSERT INTO testBug17857 VALUES (?)");
+			pStmt = this.conn
+					.prepareStatement("INSERT INTO testBug17857 VALUES (?)");
 			pStmt.close();
 			try {
 				pStmt.clearParameters();
 			} catch (SQLException sqlEx) {
 				assertEquals("08003", sqlEx.getSQLState());
 			}
-			
-			pStmt = ((com.mysql.jdbc.Connection)this.conn).clientPrepareStatement("INSERT INTO testBug17857 VALUES (?)");
+
+			pStmt = ((com.mysql.jdbc.Connection) this.conn)
+					.clientPrepareStatement("INSERT INTO testBug17857 VALUES (?)");
 			pStmt.close();
 			try {
 				pStmt.clearParameters();
 			} catch (SQLException sqlEx) {
 				assertEquals("08003", sqlEx.getSQLState());
 			}
-			
+
 		} finally {
 			if (pStmt != null) {
 				pStmt.close();
 			}
 		}
 	}
-	
+
 	/**
-	 * Tests fix for BUG#18740 - Data truncation and getWarnings()
-	 * only returns last warning in set.
+	 * Tests fix for BUG#18740 - Data truncation and getWarnings() only returns
+	 * last warning in set.
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug18740() throws Exception {
 		if (!versionMeetsMinimum(5, 0, 2)) {
 			createTable("testWarnings", "(field1 smallint(6),"
-					+ "field2 varchar(6),"
-					+ "UNIQUE KEY field1(field1))");
-	
+					+ "field2 varchar(6)," + "UNIQUE KEY field1(field1))");
+
 			try {
 				this.stmt.executeUpdate("INSERT INTO testWarnings VALUES "
-					+ "(10001, 'data1'),"
-					+ "(10002, 'data2 foo'),"
-					+ "(10003, 'data3'),"
-					+ "(10004999, 'data4'),"
-					+ "(10005, 'data5')");
+						+ "(10001, 'data1')," + "(10002, 'data2 foo'),"
+						+ "(10003, 'data3')," + "(10004999, 'data4'),"
+						+ "(10005, 'data5')");
 			} catch (SQLException sqlEx) {
 				assertEquals("01004", sqlEx.getSQLState());
 				assertEquals("01004", sqlEx.getNextException().getSQLState());
-				
+
 				SQLWarning sqlWarn = this.stmt.getWarnings();
 				assertEquals("01000", sqlWarn.getSQLState());
 				assertEquals("01000", sqlWarn.getNextWarning().getSQLState());
 			}
 		}
 	}
-	
+
 	/**
 	 * Tests fix for BUG#19615, PreparedStatement.setObject(int, Object, int)
 	 * doesn't respect scale of BigDecimals.
 	 * 
-	 * @throws Exception if the test fails.
+	 * @throws Exception
+	 *             if the test fails.
 	 */
 	public void testBug19615() throws Exception {
 		createTable("testBug19615", "(field1 DECIMAL(19, 12))");
-		
+
 		try {
 			BigDecimal dec = new BigDecimal("1.234567");
-			
-			this.pstmt = this.conn.prepareStatement("INSERT INTO testBug19615 VALUES (?)");
+
+			this.pstmt = this.conn
+					.prepareStatement("INSERT INTO testBug19615 VALUES (?)");
 			this.pstmt.setObject(1, dec, Types.DECIMAL);
 			this.pstmt.executeUpdate();
 			this.pstmt.close();
-			
-			
+
 			this.rs = this.stmt.executeQuery("SELECT field1 FROM testBug19615");
 			this.rs.next();
 			assertEquals(dec, this.rs.getBigDecimal(1).setScale(6));
 			this.rs.close();
 			this.stmt.executeUpdate("TRUNCATE TABLE testBug19615");
-			
-			this.pstmt = ((com.mysql.jdbc.Connection)this.conn).clientPrepareStatement("INSERT INTO testBug19615 VALUES (?)");
+
+			this.pstmt = ((com.mysql.jdbc.Connection) this.conn)
+					.clientPrepareStatement("INSERT INTO testBug19615 VALUES (?)");
 			this.pstmt.setObject(1, dec, Types.DECIMAL);
 			this.pstmt.executeUpdate();
 			this.pstmt.close();
-			
-			
+
 			this.rs = this.stmt.executeQuery("SELECT field1 FROM testBug19615");
 			this.rs.next();
 			assertEquals(dec, this.rs.getBigDecimal(1).setScale(6));
@@ -3157,9 +3181,13 @@ public class StatementRegressionTest extends BaseTestCase {
 		} finally {
 			closeMemberJDBCResources();
 		}
-		
 	}
-
+	
+	/**
+	 * Tests fix for BUG#20029 - NPE thrown from executeBatch().
+	 * 
+	 * @throws Exception
+	 */
 	public void testBug20029() throws Exception {
 		createTable("testBug20029", ("(field1 int)"));
 		
@@ -3218,6 +3246,49 @@ public class StatementRegressionTest extends BaseTestCase {
 		}
 	}
 	
+	/**
+	 * Fixes BUG#20687 - Can't pool server-side prepared statements, exception
+	 * raised when re-using them.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug20687() throws Exception {
+		if (!isRunningOnJdk131() && versionMeetsMinimum(5, 0)) {
+			createTable("testBug20687", "(field1 int)");
+			Connection poolingConn = null;
+			
+			Properties props = new Properties();
+			props.setProperty("cachePrepStmts", "true");
+			
+			PreparedStatement pstmt1 = null;
+			PreparedStatement pstmt2  = null;
+			
+			try {
+				poolingConn = getConnectionWithProps(props);
+				pstmt1 = poolingConn.prepareStatement("SELECT field1 FROM testBug20687");
+				pstmt1.executeQuery();
+				pstmt1.close();
+				
+				pstmt2 = poolingConn.prepareStatement("SELECT field1 FROM testBug20687");
+				pstmt2.executeQuery();
+				assertTrue(pstmt1 == pstmt2);
+				pstmt2.close();
+			} finally {
+				if (pstmt1 != null) {
+					pstmt1.close();
+				}
+				
+				if (pstmt2 != null) {
+					pstmt2.close();
+				}
+				
+				if (poolingConn != null) {
+					poolingConn.close();
+				}
+			}
+		}
+	}
+	
 	public void testLikeWithBackslashes() throws Exception {
 		if (!versionMeetsMinimum(5, 0, 0)) {
 			return;
@@ -3265,45 +3336,33 @@ public class StatementRegressionTest extends BaseTestCase {
 	}
 
 	/**
-	 * Fixes BUG#20687 - Can't pool server-side prepared statements, exception
-	 * raised when re-using them.
-	 * 
+	 * Tests fix for BUG#20650 - Statement.cancel() causes NullPointerException
+     * if underlying connection has been closed due to server failure.
+     * 
 	 * @throws Exception if the test fails.
 	 */
-	public void testBug20687() throws Exception {
-		if (!isRunningOnJdk131() && versionMeetsMinimum(4, 1)) {
-			createTable("testBug20687", "(field1 int)");
-			Connection poolingConn = null;
-			
-			Properties props = new Properties();
-			props.setProperty("cachePrepStmts", "true");
-			props.setProperty("prepStmtCacheSqlLimit", "5000");
-			
-			StringBuffer queryBuf = new StringBuffer("SELECT field1 FROM testBug20687 /*");
-			for (int i = 0; i < 1000; i++) {
-				queryBuf.append("a");
+	public void testBug20650() throws Exception {
+		Connection closedConn = null;
+		Statement cancelStmt = null;
+		
+		try {
+			closedConn = getConnectionWithProps(null);
+			cancelStmt = closedConn.createStatement();
+		
+			closedConn.close();
+
+			cancelStmt.cancel();
+		} finally {
+			if (cancelStmt != null) {
+				cancelStmt.close();
 			}
 			
-			queryBuf.append(" */");
-			
-			String query = queryBuf.toString();
-			
-			try {
-				poolingConn = getConnectionWithProps(props);
-				PreparedStatement pstmt1 = poolingConn.prepareStatement(query);
-				pstmt1.executeQuery();
-				pstmt1.close();
-				
-				PreparedStatement pstmt2 = poolingConn.prepareStatement(query);
-				pstmt2.executeQuery();
-				assertSame(pstmt1, pstmt2);
-				pstmt2.close();
-			} finally {
-				
+			if (closedConn != null && !closedConn.isClosed()) {
+				closedConn.close();
 			}
 		}
 	}
-	
+
 	/**
 	 * Tests fix for BUG#20888 - escape of quotes in client-side prepared 
 	 * statements parsing not respected.
@@ -3315,7 +3374,7 @@ public class StatementRegressionTest extends BaseTestCase {
 		try {
 			String s = "SELECT 'What do you think about D\\'Artanian''?', \"What do you think about D\\\"Artanian\"\"?\"";
 			this.pstmt = ((com.mysql.jdbc.Connection)this.conn).clientPrepareStatement(s);
-
+			
 			this.rs = this.pstmt.executeQuery();
 			this.rs.next();
 			assertEquals(this.rs.getString(1), "What do you think about D'Artanian'?");
@@ -3324,7 +3383,7 @@ public class StatementRegressionTest extends BaseTestCase {
 			closeMemberJDBCResources();
 		}
 	}
-	
+
 	/**
 	 * Tests Bug#21207 - Driver throws NPE when tracing prepared statements that
 	 * have been closed (in asSQL()).
@@ -3340,7 +3399,39 @@ public class StatementRegressionTest extends BaseTestCase {
 			closeMemberJDBCResources();
 		}
 	}
-
+	
+	/**
+	 * Tests fix for BUG#22359 - Driver was using millis for
+	 * Statement.setQueryTimeout() when spec says argument is
+	 * seconds.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug22359() throws Exception {
+		if (versionMeetsMinimum(5, 0)) {
+			Statement timeoutStmt = null;
+			
+			try {
+				timeoutStmt = this.conn.createStatement();
+				timeoutStmt.setQueryTimeout(2);
+				
+				long begin = System.currentTimeMillis();
+				
+				try {
+					timeoutStmt.execute("SELECT SLEEP(30)");
+				} catch (MySQLTimeoutException timeoutEx) {
+					long end = System.currentTimeMillis();
+					
+					assertTrue((end - begin) > 1000);
+				}
+			} finally {
+				if (timeoutStmt != null) {
+					timeoutStmt.close();
+				}
+			}
+		}
+	}
+	
 	/**
 	 * Tests fix for BUG#22290 - Driver issues truncation on write exception when
 	 * it shouldn't (due to sending big decimal incorrectly to server with
@@ -3360,7 +3451,7 @@ public class StatementRegressionTest extends BaseTestCase {
 				this.stmt
 						.executeUpdate("INSERT INTO testbug22290 (`id`,`cost`) VALUES (1,'1.00')"),
 				1);
-	
+
 		Connection configuredConn = null;
 		
 		try {
@@ -3389,7 +3480,7 @@ public class StatementRegressionTest extends BaseTestCase {
 			}
 		}
 	}
-	
+
 	public void testClientPreparedSetBoolean() throws Exception {
 		try {
 			this.pstmt = ((com.mysql.jdbc.Connection)this.conn).clientPrepareStatement("SELECT ?");
@@ -3403,4 +3494,5 @@ public class StatementRegressionTest extends BaseTestCase {
 			closeMemberJDBCResources();
 		}
 	}
+
 }
