@@ -27,12 +27,14 @@ package testsuite.simple;
 import java.io.ByteArrayInputStream;
 import java.io.CharArrayReader;
 import java.io.Reader;
+import java.rmi.server.UID;
 import java.sql.CallableStatement;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
+import java.sql.Types;
 import java.util.Properties;
 
 import testsuite.BaseTestCase;
@@ -44,7 +46,8 @@ import com.mysql.jdbc.SQLError;
  * DOCUMENT ME!
  * 
  * @author Mark Matthews
- * @version $Id: StatementsTest.java 5034 2006-03-09 20:56:13Z mmatthews $
+ * @version $Id: StatementsTest.java 4494 2005-10-31 22:30:34 -0600 (Mon, 31 Oct
+ *          2005) mmatthews $
  */
 public class StatementsTest extends BaseTestCase {
 	private static final int MAX_COLUMN_LENGTH = 255;
@@ -271,54 +274,56 @@ public class StatementsTest extends BaseTestCase {
 	 *             DOCUMENT ME!
 	 */
 	public void testAutoIncrement() throws SQLException {
-		try {
-			this.stmt = this.conn.createStatement(
-					java.sql.ResultSet.TYPE_FORWARD_ONLY,
-					java.sql.ResultSet.CONCUR_READ_ONLY);
-			this.stmt.setFetchSize(Integer.MIN_VALUE);
-			this.stmt
-					.executeUpdate("INSERT INTO statement_test (strdata1) values ('blah')");
+		if (!isRunningOnJdk131()) {
+			try {
+				this.stmt = this.conn.createStatement(
+						java.sql.ResultSet.TYPE_FORWARD_ONLY,
+						java.sql.ResultSet.CONCUR_READ_ONLY);
+				this.stmt.setFetchSize(Integer.MIN_VALUE);
+				this.stmt
+						.executeUpdate("INSERT INTO statement_test (strdata1) values ('blah')");
 
-			int autoIncKeyFromApi = -1;
-			this.rs = this.stmt.getGeneratedKeys();
+				int autoIncKeyFromApi = -1;
+				this.rs = this.stmt.getGeneratedKeys();
 
-			if (this.rs.next()) {
-				autoIncKeyFromApi = this.rs.getInt(1);
-			} else {
-				fail("Failed to retrieve AUTO_INCREMENT using Statement.getGeneratedKeys()");
-			}
-
-			this.rs.close();
-
-			int autoIncKeyFromFunc = -1;
-			this.rs = this.stmt.executeQuery("SELECT LAST_INSERT_ID()");
-
-			if (this.rs.next()) {
-				autoIncKeyFromFunc = this.rs.getInt(1);
-			} else {
-				fail("Failed to retrieve AUTO_INCREMENT using LAST_INSERT_ID()");
-			}
-
-			if ((autoIncKeyFromApi != -1) && (autoIncKeyFromFunc != -1)) {
-				assertTrue(
-						"Key retrieved from API ("
-								+ autoIncKeyFromApi
-								+ ") does not match key retrieved from LAST_INSERT_ID() "
-								+ autoIncKeyFromFunc + ") function",
-						autoIncKeyFromApi == autoIncKeyFromFunc);
-			} else {
-				fail("AutoIncrement keys were '0'");
-			}
-		} finally {
-			if (this.rs != null) {
-				try {
-					this.rs.close();
-				} catch (Exception ex) { /* ignore */
-					;
+				if (this.rs.next()) {
+					autoIncKeyFromApi = this.rs.getInt(1);
+				} else {
+					fail("Failed to retrieve AUTO_INCREMENT using Statement.getGeneratedKeys()");
 				}
-			}
 
-			this.rs = null;
+				this.rs.close();
+
+				int autoIncKeyFromFunc = -1;
+				this.rs = this.stmt.executeQuery("SELECT LAST_INSERT_ID()");
+
+				if (this.rs.next()) {
+					autoIncKeyFromFunc = this.rs.getInt(1);
+				} else {
+					fail("Failed to retrieve AUTO_INCREMENT using LAST_INSERT_ID()");
+				}
+
+				if ((autoIncKeyFromApi != -1) && (autoIncKeyFromFunc != -1)) {
+					assertTrue(
+							"Key retrieved from API ("
+									+ autoIncKeyFromApi
+									+ ") does not match key retrieved from LAST_INSERT_ID() "
+									+ autoIncKeyFromFunc + ") function",
+							autoIncKeyFromApi == autoIncKeyFromFunc);
+				} else {
+					fail("AutoIncrement keys were '0'");
+				}
+			} finally {
+				if (this.rs != null) {
+					try {
+						this.rs.close();
+					} catch (Exception ex) { /* ignore */
+						;
+					}
+				}
+
+				this.rs = null;
+			}
 		}
 	}
 
@@ -519,6 +524,251 @@ public class StatementsTest extends BaseTestCase {
 		}
 	}
 
+	public void testCancelStatement() throws Exception {
+
+		if (versionMeetsMinimum(5, 0)) {
+			Connection cancelConn = null;
+	
+			try {
+				cancelConn = getConnectionWithProps(null);
+				final Statement cancelStmt = cancelConn.createStatement();
+			
+				cancelStmt.setQueryTimeout(1);
+	
+				long begin = System.currentTimeMillis();
+	
+				try {
+					cancelStmt.execute("SELECT SLEEP(30)");
+				} catch (SQLException sqlEx) {
+					assertTrue("Probably wasn't actually cancelled", System
+							.currentTimeMillis()
+							- begin < 30000);
+				}
+	
+				for (int i = 0; i < 1000; i++) {
+					try {
+						cancelStmt.executeQuery("SELECT 1");
+					} catch (SQLException timedOutEx) {
+						break;
+					}
+				}
+				
+				// Make sure we can still use the connection...
+	
+				cancelStmt.setQueryTimeout(0);
+				this.rs = cancelStmt.executeQuery("SELECT 1");
+	
+				assertTrue(this.rs.next());
+				assertEquals(1, this.rs.getInt(1));
+	
+				cancelStmt.setQueryTimeout(0);
+	
+				new Thread() {
+	
+					public void run() {
+						try {
+							try {
+								sleep(5000);
+							} catch (InterruptedException iEx) {
+								// ignore
+							}
+	
+							cancelStmt.cancel();
+						} catch (SQLException sqlEx) {
+							throw new RuntimeException(sqlEx.toString());
+						}
+					}
+	
+				}.start();
+	
+				begin = System.currentTimeMillis();
+	
+				try {
+					cancelStmt.execute("SELECT SLEEP(30)");
+				} catch (SQLException sqlEx) {
+					assertTrue("Probably wasn't actually cancelled", System
+							.currentTimeMillis()
+							- begin < 30000);
+				}
+	
+				for (int i = 0; i < 1000; i++) {
+					try {
+						cancelStmt.executeQuery("SELECT 1");
+					} catch (SQLException timedOutEx) {
+						break;
+					}
+				}
+				
+				// Make sure we can still use the connection...
+	
+				this.rs = cancelStmt.executeQuery("SELECT 1");
+	
+				assertTrue(this.rs.next());
+				assertEquals(1, this.rs.getInt(1));
+				
+				final PreparedStatement cancelPstmt = cancelConn.prepareStatement("SELECT SLEEP(30)");
+				
+				cancelPstmt.setQueryTimeout(1);
+	
+				begin = System.currentTimeMillis();
+	
+				try {
+					cancelPstmt.execute();
+				} catch (SQLException sqlEx) {
+					assertTrue("Probably wasn't actually cancelled", System
+							.currentTimeMillis()
+							- begin < 30000);
+				}
+	
+				for (int i = 0; i < 1000; i++) {
+					try {
+						cancelPstmt.executeQuery("SELECT 1");
+					} catch (SQLException timedOutEx) {
+						break;
+					}
+				}
+				
+				// Make sure we can still use the connection...
+	
+				this.rs = cancelStmt.executeQuery("SELECT 1");
+	
+				assertTrue(this.rs.next());
+				assertEquals(1, this.rs.getInt(1));
+	
+				cancelPstmt.setQueryTimeout(0);
+	
+				new Thread() {
+	
+					public void run() {
+						try {
+							try {
+								sleep(5000);
+							} catch (InterruptedException iEx) {
+								// ignore
+							}
+	
+							
+							cancelPstmt.cancel();
+						} catch (SQLException sqlEx) {
+							throw new RuntimeException(sqlEx.toString());
+						}
+					}
+	
+				}.start();
+	
+				begin = System.currentTimeMillis();
+	
+				try {
+					cancelPstmt.execute();
+				} catch (SQLException sqlEx) {
+					assertTrue("Probably wasn't actually cancelled", System
+							.currentTimeMillis()
+							- begin < 30000);
+				}
+	
+				for (int i = 0; i < 1000; i++) {
+					try {
+						cancelPstmt.executeQuery("SELECT 1");
+					} catch (SQLException timedOutEx) {
+						break;
+					}
+				}
+				
+				// Make sure we can still use the connection...
+	
+				this.rs = cancelStmt.executeQuery("SELECT 1");
+	
+				assertTrue(this.rs.next());
+				assertEquals(1, this.rs.getInt(1));
+				
+				final PreparedStatement cancelClientPstmt = ((com.mysql.jdbc.Connection)cancelConn).clientPrepareStatement("SELECT SLEEP(30)");
+				
+				cancelClientPstmt.setQueryTimeout(1);
+	
+				begin = System.currentTimeMillis();
+	
+				try {
+					cancelClientPstmt.execute();
+				} catch (SQLException sqlEx) {
+					assertTrue("Probably wasn't actually cancelled", System
+							.currentTimeMillis()
+							- begin < 30000);
+				}
+	
+				for (int i = 0; i < 1000; i++) {
+					try {
+						cancelStmt.executeQuery("SELECT 1");
+					} catch (SQLException timedOutEx) {
+						break;
+					}
+				}
+				
+				// Make sure we can still use the connection...
+	
+				this.rs = cancelStmt.executeQuery("SELECT 1");
+	
+				assertTrue(this.rs.next());
+				assertEquals(1, this.rs.getInt(1));
+	
+				cancelClientPstmt.setQueryTimeout(0);
+	
+				new Thread() {
+	
+					public void run() {
+						try {
+							try {
+								sleep(5000);
+							} catch (InterruptedException iEx) {
+								// ignore
+							}
+	
+							
+							cancelClientPstmt.cancel();
+						} catch (SQLException sqlEx) {
+							throw new RuntimeException(sqlEx.toString());
+						}
+					}
+	
+				}.start();
+	
+				begin = System.currentTimeMillis();
+	
+				try {
+					cancelClientPstmt.execute();
+				} catch (SQLException sqlEx) {
+					assertTrue("Probably wasn't actually cancelled", System
+							.currentTimeMillis()
+							- begin < 30000);
+				}
+	
+				for (int i = 0; i < 1000; i++) {
+					try {
+						cancelClientPstmt.executeQuery("SELECT 1");
+					} catch (SQLException timedOutEx) {
+						break;
+					}
+				}
+				
+				// Make sure we can still use the connection...
+	
+				this.rs = cancelStmt.executeQuery("SELECT 1");
+	
+				assertTrue(this.rs.next());
+				assertEquals(1, this.rs.getInt(1));
+			} finally {
+				if (this.rs != null) {
+					ResultSet toClose = this.rs;
+					this.rs = null;
+					toClose.close();
+				}
+	
+				if (cancelConn != null) {
+					cancelConn.close();
+				}
+			}
+		}
+	}
+
 	/**
 	 * DOCUMENT ME!
 	 * 
@@ -683,25 +933,28 @@ public class StatementsTest extends BaseTestCase {
 						+ "'", (updateCount == 1));
 			}
 
-			this.stmt
-					.executeUpdate("INSERT INTO statement_test (strdata1, strdata2) values ('a', 'a'), ('b', 'b'), ('c', 'c')");
-			this.rs = this.stmt.getGeneratedKeys();
+			if (!isRunningOnJdk131()) {
+				int insertIdFromGeneratedKeys = Integer.MIN_VALUE;
 
-			if (this.rs.next()) {
-				this.rs.getInt(1);
+				this.stmt
+						.executeUpdate("INSERT INTO statement_test (strdata1, strdata2) values ('a', 'a'), ('b', 'b'), ('c', 'c')");
+				this.rs = this.stmt.getGeneratedKeys();
+
+				if (this.rs.next()) {
+					insertIdFromGeneratedKeys = this.rs.getInt(1);
+				}
+
+				this.rs.close();
+				this.rs = this.stmt.executeQuery("SELECT LAST_INSERT_ID()");
+
+				int insertIdFromServer = Integer.MIN_VALUE;
+
+				if (this.rs.next()) {
+					insertIdFromServer = this.rs.getInt(1);
+				}
+
+				assertEquals(insertIdFromGeneratedKeys, insertIdFromServer);
 			}
-
-			this.rs.close();
-			this.rs = this.stmt.executeQuery("SELECT LAST_INSERT_ID()");
-
-			int updateCountFromServer = 0;
-
-			if (this.rs.next()) {
-				updateCountFromServer = this.rs.getInt(1);
-			}
-
-			System.out.println("Update count from server: "
-					+ updateCountFromServer);
 		} finally {
 			if (this.rs != null) {
 				try {
@@ -913,6 +1166,131 @@ public class StatementsTest extends BaseTestCase {
 		}
 	}
 
+	public void testRowFetch() throws Exception {
+		if (versionMeetsMinimum(5, 0, 5)) {
+			createTable("testRowFetch", "(field1 int)");
+
+			this.stmt.executeUpdate("INSERT INTO testRowFetch VALUES (1)");
+
+			Connection fetchConn = null;
+			
+			Properties props = new Properties();
+			props.setProperty("useCursorFetch", "true");
+			
+			
+			try {
+				fetchConn = getConnectionWithProps(props);
+				
+				PreparedStatement fetchStmt = fetchConn
+						.prepareStatement("SELECT field1 FROM testRowFetch WHERE field1=1");
+				fetchStmt.setFetchSize(10);
+				this.rs = fetchStmt.executeQuery();
+				assertTrue(this.rs.next());
+	
+				this.stmt.executeUpdate("INSERT INTO testRowFetch VALUES (2), (3)");
+	
+				fetchStmt = fetchConn
+						.prepareStatement("SELECT field1 FROM testRowFetch ORDER BY field1");
+				fetchStmt.setFetchSize(1);
+				this.rs = fetchStmt.executeQuery();
+	
+				assertTrue(this.rs.next());
+				assertEquals(1, this.rs.getInt(1));
+				assertTrue(this.rs.next());
+				assertEquals(2, this.rs.getInt(1));
+				assertTrue(this.rs.next());
+				assertEquals(3, this.rs.getInt(1));
+				assertEquals(false, this.rs.next());
+	
+				fetchStmt.executeQuery();
+			} finally {
+				if (fetchConn != null) {
+					fetchConn.close();
+				}
+			}
+
+		}
+	}
+
+	/**
+	 * DOCUMENT ME!
+	 * 
+	 * @throws SQLException
+	 *             DOCUMENT ME!
+	 */
+	public void testSelectColumns() throws SQLException {
+		for (int i = 6; i < MAX_COLUMNS_TO_TEST; i += STEP) {
+			long start = System.currentTimeMillis();
+			this.rs = this.stmt
+					.executeQuery("SELECT * from statement_col_test_" + i);
+
+			if (this.rs.next()) {
+				;
+			}
+
+			long end = System.currentTimeMillis();
+			System.out.println(i + " columns = " + (end - start) + " ms");
+		}
+	}
+
+	/**
+	 * Tests for PreparedStatement.setObject()
+	 * 
+	 * @throws Exception
+	 */
+	public void testSetObject() throws Exception {
+		Properties props = new Properties();
+		props.put("noDatetimeStringSync", "true"); // value=true for #5
+		Connection conn1 = getConnectionWithProps(props);
+		Statement stmt1 = conn1.createStatement();
+		stmt1.executeUpdate("DROP TABLE IF EXISTS t1");
+		stmt1.executeUpdate("CREATE TABLE t1 (" + "c1 DECIMAL," // instance of
+																// String
+				+ "c2 VARCHAR(255)," // instance of String
+				+ "c3 BLOB," // instance of byte[]
+				+ "c4 DATE," // instance of java.util.Date
+				+ "c5 TIMESTAMP," // instance of String
+				+ "c6 TIME," // instance of String
+				+ "c7 TIME)"); // instance of java.sql.Timestamp
+
+		this.pstmt = conn1
+				.prepareStatement("INSERT INTO t1 VALUES (?, ?, ?, ?, ?, ?, ?)");
+
+		long currentTime = System.currentTimeMillis();
+
+		this.pstmt.setObject(1, "1000", Types.DECIMAL);
+		this.pstmt.setObject(2, "2000", Types.VARCHAR);
+		this.pstmt.setObject(3, new byte[] { 0 }, Types.BLOB);
+		this.pstmt.setObject(4, new java.util.Date(currentTime), Types.DATE);
+		this.pstmt.setObject(5, "2000-01-01 23-59-59", Types.TIMESTAMP);
+		this.pstmt.setObject(6, "11:22:33", Types.TIME);
+		this.pstmt
+				.setObject(7, new java.sql.Timestamp(currentTime), Types.TIME);
+		this.pstmt.execute();
+		this.rs = stmt1.executeQuery("SELECT * FROM t1");
+		this.rs.next();
+
+		assertEquals("1000", this.rs.getString(1));
+		assertEquals("2000", this.rs.getString(2));
+		assertEquals(1, ((byte[]) this.rs.getObject(3)).length);
+		assertEquals(0, ((byte[]) this.rs.getObject(3))[0]);
+		assertEquals(new java.sql.Date(currentTime).toString(), this.rs
+				.getDate(4).toString());
+
+		if (versionMeetsMinimum(4, 1)) {
+			assertEquals("2000-01-01 23:59:59", this.rs.getString(5));
+		} else {
+			assertEquals("20000101235959", this.rs.getString(5));
+		}
+
+		assertEquals("11:22:33", this.rs.getString(6));
+		assertEquals(new java.sql.Time(currentTime).toString(), this.rs
+				.getString(7));
+	}
+
+	// Server-side prepared statements can only reset streamed data
+	// in-toto, not piecemiel.
+
 	public void testStatementRewriteBatch() throws SQLException {
 		Properties props = new Properties();
 		props.setProperty("rewriteBatchedStatements", "true");
@@ -1014,27 +1392,6 @@ public class StatementsTest extends BaseTestCase {
 			assertEquals(i, genKeys.getInt(1));
 		}
 	}
-	
-	/**
-	 * DOCUMENT ME!
-	 * 
-	 * @throws SQLException
-	 *             DOCUMENT ME!
-	 */
-	public void testSelectColumns() throws SQLException {
-		for (int i = 6; i < MAX_COLUMNS_TO_TEST; i += STEP) {
-			long start = System.currentTimeMillis();
-			this.rs = this.stmt
-					.executeQuery("SELECT * from statement_col_test_" + i);
-
-			if (this.rs.next()) {
-				;
-			}
-
-			long end = System.currentTimeMillis();
-			System.out.println(i + " columns = " + (end - start) + " ms");
-		}
-	}
 
 	public void testStreamChange() throws Exception {
 		createTable("testStreamChange",
@@ -1095,16 +1452,15 @@ public class StatementsTest extends BaseTestCase {
 	 *             DOCUMENT ME!
 	 */
 	public void testStubbed() throws SQLException {
-		try {
-			this.stmt.getResultSetHoldability();
-		} catch (NotImplemented notImplEx) {
-			;
+		if (!isRunningOnJdk131()) {
+			try {
+				this.stmt.getResultSetHoldability();
+			} catch (NotImplemented notImplEx) {
+				;
+			}
 		}
 	}
-
-	// Server-side prepared statements can only reset streamed data
-	// in-toto, not piecemiel.
-
+	
 	public void testTruncationOnRead() throws Exception {
 		this.rs = this.stmt.executeQuery("SELECT '" + Long.MAX_VALUE + "'");
 		this.rs.next();
